/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <dirent.h>
#include <iostream>

#include "Macros.h"
#include "Utilities.h"
#include "AutoSlide.h"


using namespace slideshow;


AutoSlide::AutoSlide (int video_mode) :
	Presentation (video_mode)
{
}


AutoSlide::~AutoSlide()
{
}


void
AutoSlide::Load (const char *path, bool recursive)
{
	if (!path) return;
	
	LoadRecursiveIterarion (path, recursive);
}

void
AutoSlide::LoadRecursiveIterarion (const char *path, bool recursive)
{
	struct dirent **filelist;

	int num = scandir(path, &filelist, 0, alphasort);
	if (num < 0) {
		cerr << "Error reading " << path << endl;
		return;
	}

	while (num--) {
		string fullname, slash("/");
		fullname = path + slash + filelist[num]->d_name;
		
		if (recursive && \
		    filelist[num]->d_type == DT_DIR && \
		    strcmp (filelist[num]->d_name, ".") && \
		    strcmp (filelist[num]->d_name, ".."))
		{
			LoadRecursiveIterarion ((char*)fullname.c_str(), recursive);
		} else {
			AddFile (fullname);
		}
		free (filelist[num]);
	}
	free(filelist);
}


void
AutoSlide::AddFile (string filename)
{
	Drawable *smthng = NULL;
	
	if (endsWith (filename, ".jpg") ||
	    endsWith (filename, ".png") ||
	    endsWith (filename, ".gif") ||
	    endsWith (filename, ".bmp") ||
	    endsWith (filename, ".pnm") ||
	    endsWith (filename, ".xpm") ||
	    endsWith (filename, ".xcf") ||
	    endsWith (filename, ".pcx") ||
	    endsWith (filename, ".tif") ||
	    endsWith (filename, ".lbm")) {

		smthng = new Image();
		static_cast<Image*>(smthng)->Load(filename.c_str());

	} else if (endsWith (filename, ".mpg")) {
		
		smthng = new Mpeg();
		static_cast<Mpeg*>(smthng)->Load(filename.c_str());
	}


	if (smthng) {
		Page *page = new Page();
		Eraser *eraser = new Eraser();

		page->Add(eraser);
		page->Add(smthng);

		smthng->AlignCenter();
			
		Add(page);
	}
}
