/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*
**  bmp2png --- conversion from (Windows or OS/2 style) BMP to PNG
**  png2bmp --- conversion from PNG to (Windows style) BMP
**
**  Copyright (C) 1999-2002 MIYASAKA Masaru
**
**  For conditions of distribution and use,
**  see copyright notice in common.h.
*/

#include "Bmp2Png.h"

#include <iostream>
using namespace std;


#if defined(__DJGPP__)		/* DJGPP V.2 */
#include <crt0.h>
int _crt0_startup_flags = _CRT0_FLAG_DISALLOW_RESPONSE_FILES;
unsigned short _djstat_flags =	/* for stat() */
_STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC | _STAT_DIRSIZE |
_STAT_ROOT_TIME;
#endif

#if defined(__BORLANDC__)	/* Borland C++ */
#include <wildargs.h>
typedef void _RTLENTRY (* _RTLENTRY _argv_expand_fnc)(char *, _PFN_ADDARG);
typedef void _RTLENTRY (* _RTLENTRY _wargv_expand_fnc)(wchar_t *, _PFN_ADDARG);
_argv_expand_fnc  _argv_expand_ptr  = _expand_wild;		/* expand wild cards */
_wargv_expand_fnc _wargv_expand_ptr = _wexpand_wild;
#endif


/* -----------------------------------------------------------------------
**		screen management
*/

#define LINE_LEN	79
#define STAT_LEN	22
#define DOTS_MAX	(LINE_LEN - STAT_LEN - 1)

static char status_msg[320];
static int  scaleflag = 0;
static int  maxdots   = 0;
static int  currdots  = -1;

int quietmode = 0;
int errorlog  = 0;



/* -----------------------------------------------------------------------
**		libpng progress meter
*/

/* --------------------------------------------------- *

PNG pass width height origin area progress
0   1/8   1/8   (0,0)  1/64  1/64
1   1/8   1/8   (4,0)  1/64  1/32
2   1/4   1/8   (0,4)  1/32  1/16
3   1/4   1/4   (2,0)  1/16  1/8
4   1/2   1/4   (0,2)  1/8   1/4
5   1/2   1/2   (1,0)  1/4   1/2
6    1    1/2   (0,1)  1/2    1

* --------------------------------------------------- */

static png_uint_32 counter;
static png_uint_32 maxcount;
static int         scaledots;


static png_uint_32
maxcount_adam7(png_uint_32 width, png_uint_32 height)
{
	   png_uint_32 c = 0;

	   if (    1    ) c += ((height - 0 + 7) / 8) * 1;		/* Pass 0 */
	   if (width > 4) c += ((height - 0 + 7) / 8) * 1;		/* Pass 1 */
	   if (    1    ) c += ((height - 4 + 7) / 8) * 2;		/* Pass 2 */
	   if (width > 2) c += ((height - 0 + 3) / 4) * 2;		/* Pass 3 */
	   if (    1    ) c += ((height - 2 + 3) / 4) * 4;		/* Pass 4 */
	   if (width > 1) c += ((height - 0 + 1) / 2) * 4;		/* Pass 5 */
	   if (    1    ) c += ((height - 1 + 1) / 2) * 8;		/* Pass 6 */

	   return c;
}


/*
**		initialize the progress meter
*/
void init_progress_meter(png_structp png_ptr, png_uint_32 width,
                         png_uint_32 height)
{
	   enum { W = 1024, H = 768 };

	   if (png_set_interlace_handling(png_ptr) == 7) {
			 maxcount = maxcount_adam7(width, height);	/* interlace */
	   } else {
			 maxcount = height;							/* non-interlace */
	   }
	   if (height > ((png_uint_32)W * H) / width) {
			 scaledots = DOTS_MAX;
	   } else {
			 scaledots = (DOTS_MAX * width * height + (W * H - 1)) / (W * H);
	   }
	   counter = 0;
//	   init_progress(scaledots);
}


/*
**		row callback function for progress meter
*/
void row_callback(png_structp png_ptr, png_uint_32 row, int pass)
{
/*	static const png_byte step[] = { 1, 1, 2, 2, 4, 4, 8 }; */

	   if (row == 0) pass--;
	   /*
	    *	libpng's bug ?? : In the case of interlaced image,
	    *	  this function is called with 0(row) and currpass+1(pass)
	    *	  when the row should equal to height and pass should equal
	    *	  to currpass.
	    */

	   counter += (1 << (pass >> 1));	/* step[pass]; */
//	   update_progress(scaledots * counter / maxcount);
}


/* -----------------------------------------------------------------------
**		libpng error handling
*/

/*
**		fatal error handling function
*/
void png_my_error(png_structp png_ptr, png_const_charp message)
{
	cerr << "ERROR (libpng): " << message << " - " << (char *)png_get_error_ptr(png_ptr) << endl;
	longjmp(png_ptr->jmpbuf, 1);
}


/*
**		non-fatal error handling function
*/
void png_my_warning(png_structp png_ptr, png_const_charp message)
{
	cerr << "WARNING (libpng): " << message << " - " << (char *)png_get_error_ptr(png_ptr) << endl;
}


/* -----------------------------------------------------------------------
**		image buffer management
*/

/*
**		allocate image buffer
*/
BOOL
imgalloc(IMAGE *img)
{
	   BYTE *bp, **rp;
	   LONG n;

	   if (img->palnum > 0) {
			 img->palette = (PALETTE*)malloc((size_t)img->palnum * sizeof(PALETTE));
			 if (img->palette == NULL) return FALSE;
	   } else {
			 img->palette = NULL;
	   }
	   img->rowbytes = ((DWORD)img->width * img->pixdepth + 31) / 32 * 4;
	   img->imgbytes = img->rowbytes * img->height;
	   img->rowptr   = (BYTE**) malloc((size_t)img->height * sizeof(BYTE *));
	   img->bmpbits  = (BYTE*) malloc((size_t)img->imgbytes);

	   if (img->rowptr == NULL || img->bmpbits == NULL) {
			 imgfree(img);
			 return FALSE;
	   }

	   n  = img->height;
	   rp = img->rowptr;
	   bp = img->bmpbits;

	   if (img->topdown) {
			 while (--n >= 0) {
				    *(rp++) = bp;
				    bp += img->rowbytes;
				    /*	((DWORD *)bp)[-1] = 0;	*/
			 }
	   } else {	/* bottom-up */
			 bp += img->imgbytes;
			 while (--n >= 0) {
				    /* fill zeros to padding bytes (for write_bmp()) */
				    ((DWORD *)bp)[-1] = 0;
				    bp -= img->rowbytes;
				    *(rp++) = bp;
			 }
	   }

	   return TRUE;
}


/*
**		free image buffer allocated by imgalloc()
*/
void
imgfree(IMAGE *img)
{
	   free(img->palette);
	   free(img->rowptr);
	   free(img->bmpbits);
}


#define isoption(p)	(IsOptChar((p)[0]) && (p)[1]!='\0')


char **
envargv(int *argcp, char ***argvp, const char *envn)
{
	   int argc, nagc, envc, i;
	   char **argv, **nagv, *envs, *ep;

	   ep = getenv(envn);
	   if (ep == NULL || ep[0] == '\0') return NULL;

	   envs = (char*)malloc(strlen(ep) + 1);
	   if (envs == NULL) return NULL;
	   strcpy(envs, ep);

	   envc = tokenize(envs, envs);
	   if (envc == 0) { free(envs); return NULL; }

	   argc = *argcp;
	   argv = *argvp;
	   nagv = (char**) malloc((argc + envc + 1) * sizeof(char *));
	   if (nagv == NULL) { free(envs); return NULL; }

	   nagc = 1;
	   nagv[0] = argv[0];

	   for (i = 0; i < envc; i++) {
			 nagv[nagc++] = envs;
			 while (*(envs++) != '\0') ;
	   }
	   for (i = 1; i < argc; i++) {
			 nagv[nagc++] = argv[i];
	   }
	   nagv[nagc] = NULL;

	   *argcp = nagc;
	   *argvp = nagv;

	   return argv;
}


int 
tokenize(char *buf, const char *str)
{
	   enum { STR = 0x01, QUOTE = 0x02 };
	   int flag = 0;
	   int num = 0;
	   char c;
	   int i;

	   while ((c = *str++) != '\0') {
			 if (!(flag & QUOTE) &&
				(c == ' ' || c == '\t' || c == '\n' || c == '\r')) {
				    if (flag & STR) {
						  flag &= ~STR;
						  *buf++ = '\0';
				    }
			 } else {
				    if (!(flag & STR)) {
						  flag |= STR;
						  num++;
				    }
				    switch (c) {
				    case '\\':
						  /*
						   *	Escaping of `"' is the same as
						   *	command-line parsing of MS-VC++.
						   *
						   *	ex.) "     ->      quote
						   *	     \"    ->      "
						   *	     \\"   -> \  + quote
						   *	     \\\"  -> \  + "
						   *	     \\\\" -> \\ + quote
						   *	     \\\\\ -> \\\\\
						   */
						  for (i = 1; (c = *str) == '\\'; str++, i++) ;
						  if (c == '"') {
								while ((i -= 2) >= 0)
									   *buf++ = '\\';
								if (i == -1) {
									   *buf++ = '"';
									   str++;
								}
						  } else {
								while ((--i) >= 0)
									   *buf++ = '\\';
						  }
						  break;

				    case '"':
						  flag ^= QUOTE;
						  break;

				    default:
						  *buf++ = c;
				    }
			 }
	   }
	   if (flag & STR) *buf = '\0';

	   return num;
}


int 
makedir(const char *path)
{
	   char dir[FILENAME_MAX];
	   struct stat sbuf;
	   char *p, c;
	   int r;

	   delslash(strcpy(dir, path));
	   if (stat(dir, &sbuf) == 0) {
			 if ((sbuf.st_mode & S_IFMT) == S_IFDIR) return 0;
			 /* errno = EEXIST; */
			 return -1;
	   }
	   p = path_skiproot(dir);
	   do {
			 p = path_nextslash(p);
			 c = *p;  *p = '\0';
			 r = MKDIR(dir, 0777);
			 *p++ = c;
	   } while (c != '\0');

	   return r;
}


int 
renbak(const char *path)
{
	   char bak[FILENAME_MAX];
	   struct stat sbuf;
	   char *sfx;
	   int i;

	   strcpy(bak, path);
	   if (stat(bak, &sbuf) != 0) return 0;

#ifdef MSDOS
	   sfx = suffix(bak);
#else
	   sfx = bak + strlen(bak);
#endif
	   strcpy(sfx, ".bak");
	   i = 0;
	   while (1) {
			 if (stat(bak, &sbuf) != 0 && rename(path, bak) == 0) return 0;
			 if (i >= 1000) break;
			 sprintf(sfx, ".%03d", i++);
	   }
	   return -1;
}


int 
cpyftime(const char *srcf, const char *dstf)
{
	   struct stat sbuf;
	   struct utimbuf ubuf;

	   if (stat(srcf, &sbuf) != 0) return -1;

	   ubuf.actime  = sbuf.st_atime;
	   ubuf.modtime = sbuf.st_mtime;

	   return utime(dstf, &ubuf);
}


/* -----------------------------------------------------------------------
**   path functions
*/

/*
**	Return a pointer that points the suffix of the PATH
**	ex.) c:\dosuty\log\test.exe -> .exe
**	ex.) c:\dosuty\log\test.tar.gz -> .gz
*/
char *
suffix(const char *path)
{
	   char c, *p, *q, *r;

	   for (r = q = p = basname(path); (c = *p) != '\0'; p++)
			 if (c == '.') q = p;
	   if (q == r) q = p;			/* dotfile with no suffix */

	   return q;
}


/*
**	Return a pointer that points the basename of the PATH
**	ex.) c:\dos\format.exe -> format.exe
*/
char *
basname(const char *path)
{
	   const char *p, *q;

	   for (p = path_skiproot(path);
		   *(q = path_nextslash(p)) != '\0'; p = q + 1) ;

	   return (char *)p;
}


/*
**	Append a path-delimiter to the PATH. If the PATH is a string
**	like "c:\", "\", "c:", "", do nothing.
**	ex.) c:\dos -> c:\dos\
*/
char *
addslash(char *path)
{
	   char *p, *q;

	   for (p = path_skiproot(path);
		   *(q = path_nextslash(p)) != '\0'; p = q + 1) ;
	   /*
	    *	s = path_skiproot( path );
	    *	if( q==s && q==p ) - s is a mull string.
	    *	if( q!=s && q==p ) - s is followed by a path delimiter.
	    *	if( q!=s && q!=p ) - s is not followed by a path delimiter.
	    */
	   if (q != p) {
			 *q++ = PATHDELIM;
			 *q   = '\0';
	   }

	   return path;
}


/*
**	Remove a path-delimiter at the end of the PATH. If the PATH is
**	a string like "c:\", "\", "c:", "", append a dot.
**	ex.) c:\dos\ -> c:\dos
**	     c:\ -> c:\.
*/
char *
delslash(char *path)
{
	   char *p, *q, *s;

	   for (p = s = path_skiproot(path);
		   *(q = path_nextslash(p)) != '\0'; p = q + 1) ;
	   /*
	    *	if( q==s && q==p ) - s is a mull string.
	    *	if( q!=s && q==p ) - s is followed by a path delimiter.
	    *	if( q!=s && q!=p ) - s is not followed by a path delimiter.
	    */
	   if (q == s) {
			 *q++ = '.';
			 *q   = '\0';
	   } else if (q == p) {
			 *--q = '\0';
	   }

	   return path;
}


char *
path_skiproot(const char *path)
{
#ifdef DRIVESUFFIX
	   if (isalpha((unsigned char)path[0])
		  && path[1] == DRIVESUFFIX) path += 2;
#endif
	   if (IsPathDelim(path[0])) path++;
	   return (char *)path;
}


char *
path_nextslash(const char *path)
{
	   char c;

	   for (; (c = *path) != '\0'; path++) {
			 if (IsDBCSLead((unsigned char)c)) {
				    if (*(++path) == '\0') break;
				    continue;
			 }
			 if (IsPathDelim(c)) break;
	   }
	   return (char *)path;
}

#ifdef WIN32_LFN

/*
**	return TRUE if the PATH is a dos-style filename.
*/
int 
is_dos_filename(const char *path)
{
	   unsigned char c;
	   char *b, *p;

	   for (b = p = basname(path); (c = *p) != '\0' && c != '.'; p++)
			 if (islower(c)) return 0;
	   if ((p - b) == 0 || (p - b) > 8) return 0;
	   if (c == '.') {
			 for (b = ++p; (c = *p) != '\0'; p++)
				    if (islower(c) || c == '.') return 0;
			 if ((p - b) == 0 || (p - b) > 3) return 0;
	   }
	   return 1;
}

#endif	/* WIN32_LFN */


/*
**  bmp2png --- conversion from (Windows or OS/2 style) BMP to PNG
**
**  Copyright (C) 1999-2002 MIYASAKA Masaru
**
**  For conditions of distribution and use,
**  see copyright notice in common.h.
*/


#define BMP2PNG_VER		"1.54 (Nov 14, 2002)"

char outnam[FILENAME_MAX];
char outdir[FILENAME_MAX];
int  deletesrc = 0;
int  copytime  = 0;
int  complevel = 6;
int  interlace = 0;
int  filters   = 0;

#if defined(WIN32) || defined(MSDOS)
char errlogfile[] = ".\\B2PERROR.LOG";
#else
char errlogfile[] = "./b2perror.log";
#endif

/* error messages */
#ifdef JAPANESE /* ---------- */
char wrn_mkdirfail[]   = "WARNING: mkdirfail\n";
char err_ropenfail[]   = "SKIPPED: read open fail - %s\n";
char err_wopenfail[]   = "SKIPPED: write open fail - %s\n";
char err_outofmemory[] = "SKIPPED: out of memory - %s\n";
/* -- */
char err_readeof[]     = "SKIPPED: reade of - %s\n";
char err_readerr[]     = "SKIPPED: read error - %s\n";
char err_not_a_bmp[]   = "SKIPPED: Not a bmp - %s\n";
char err_invalid_hed[] = "SKIPPED: Invalid BMP head - %s\n";
char err_width_zero[]  = "SKIPPED: Width zero - %s\n";
char err_height_zero[] = "SKIPPED: Height zero- %s\n";
char err_compressed[]  = "SKIPPED: compressed - %s\n";
char err_16or32bpp[]   = "SKIPPED: 16,32 bits/pixel - %s\n";
char err_invalid_bpp[] = "SKIPPED: invalid BPP - %s\n";
#else	/* ------------------- */
char wrn_mkdirfail[]   = "WARNING: Cannot create a directory - %s\n"
			 "           -> Output directory specified by '-%c' will be ignored.\n";
char err_ropenfail[]   = "SKIPPED: No such file or directory - %s\n";
char err_wopenfail[]   = "SKIPPED: Cannot create - %s\n";
char err_outofmemory[] = "SKIPPED: Out of memory - %s\n";
/* -- */
char err_readeof[]     = "SKIPPED: Premature end of BMP file - %s\n";
char err_readerr[]     = "SKIPPED: Read operation failed - %s\n";
char err_not_a_bmp[]   = "SKIPPED: Not a BMP file - %s\n";
char err_invalid_hed[] = "SKIPPED: Invalid header size in BMP file - %s\n";
char err_width_zero[]  = "SKIPPED: Invalid image size (width is less than one) - %s\n";
char err_height_zero[] = "SKIPPED: Invalid image size (height is less than one) - %s\n";
char err_compressed[]  = "SKIPPED: Compressed or bitfield BMP - %s\n";
char err_16or32bpp[]   = "SKIPPED: 16 or 32 bits/pixel BMP - %s\n";
char err_invalid_bpp[] = "SKIPPED: Invalid bit depth in BMP file - %s\n";
#endif	/* ------------------- */

static int png_filters(const char *);
static char *read_bmp(char *, FILE *, IMAGE *);
static int read_stdin(IMAGE *);
static int write_stdout(IMAGE *);
static unsigned long mgetdwl(void *);
static unsigned int mgetwl(void *);
static int write_png(char *, FILE *, IMAGE *);
static void usage_exit(char *, int);


#define elemsof(a)	(sizeof(a) / sizeof((a)[0]))


static int 
png_filters(const char *arg)
{
	   static const struct { char name[8]; int flag; } filter[] = {
			 { "NONE", PNG_FILTER_NONE  },	{ "SUB"    , PNG_FILTER_SUB   },
			 { "UP"  , PNG_FILTER_UP    },	{ "AVERAGE", PNG_FILTER_AVG   },
			 { "AVG" , PNG_FILTER_AVG   },	{ "PAETH"  , PNG_FILTER_PAETH },
			 { "ALL" , PNG_ALL_FILTERS  },
			 { "AUTO", 0                },	{ "DEFAULT", 0                }
	   };
	   char c, buf[512];
	   int i, flags = 0;

	   if (arg == NULL) return 0;				/* auto/default */

	   do {
			 i = 0;
			 while (c = *(arg++), c != ',' && c != '\0')
				    if (i < sizeof(buf) - 1) buf[i++] = toupper(c);
			 buf[i] = '\0';

			 for (i = 0; i < elemsof(filter); i++) {
				    if (strcmp(buf, filter[i].name) == 0) {
						  if (filter[i].flag == 0) flags  = 0;	/* auto/default */
						  else                     flags |= filter[i].flag;
				    }
			 }
	   } while (c != '\0');

	   return flags;
}


int 
read_file(char *fn, IMAGE *img)
{
	   FILE *fp;
	   char *p;

	   fp = fopen(fn, "rb");
	   if (fp == NULL) {
	   		fprintf (stderr, err_ropenfail, fn); 
			return 0;
	   }
	   p = read_bmp(fn, fp, img);
	   fclose(fp);
	   if (p != NULL) {
	   		fprintf (stderr, p, fn);
			return 0;
	   }
	   return 1;
}


static int 
read_stdin(IMAGE *img)
{
	   static char fn[] = " (stdin)";
	   FILE *fp;
	   char *p;

#ifdef USE_FDOPEN
	   fp = fdopen(fileno(stdin), "rb");
	   if (fp == NULL) {
	   		fprintf (stderr, err_ropenfail, fn);
			 return 0;
	   }
	   p = read_bmp(fn, fp, img);
	   fclose(fp);
#else
#ifdef USE_SETMODE
	   setmode(fileno(stdin), O_BINARY);
#endif
	   fp = stdin;
	   p = read_bmp(fn, fp, img);
#endif
	   if (p != NULL) {
			fprintf (stderr, p, fn);
			 return 0;
	   }
	   return 1;
}


int 
write_file(char *fn, IMAGE *img)
{
	   FILE *fp;
	   int b;

	   fp = fopen(fn, "wb");
	   if (fp == NULL) {
			 fprintf (stderr, err_wopenfail, fn);
			 imgfree(img);
			 return 0;
	   }
	   b = write_png(fn, fp, img);
	   fclose(fp);
	   imgfree(img);

	   return b;
}


static int 
write_stdout(IMAGE *img)
{
	   static char fn[] = " (stdout)";
	   FILE *fp;
	   int b;

#ifdef USE_FDOPEN
	   fp = fdopen(fileno(stdout), "wb");
	   if (fp == NULL) {
	   		 printf (stderr, err_wopenfail, fn);
			 imgfree(img);
			 return 0;
	   }
	   b = write_png(fn, fp, img);
	   fclose(fp);
#else
#ifdef USE_SETMODE
	   setmode(fileno(stdout), O_BINARY);
#endif
	   fp = stdout;
	   b = write_png(fn, fp, img);
#endif
	   imgfree(img);

	   return b;
}


/* -----------------------------------------------------------------------
**		BMP
*/

char 
*read_bmp(char *fn, FILE *fp, IMAGE *img)
{
	   BYTE bfh[FILEHED_SIZE + BMPV5HED_SIZE];
	   BYTE *const bih = bfh + FILEHED_SIZE;
	   BYTE rgbq[RGBQUAD_SIZE];
	   DWORD offbits, bihsize, skip;
	   DWORD compression, palsize;
	   PALETTE *pal;
	   UINT i;

	   // set_status("Reading %s", basname(fn));

	   /* ------------------------------------------------------ */

	   for (i = 1;; i--) {		/* skip macbinary header */
			 if (fread(bfh, (FILEHED_SIZE + BIHSIZE_SIZE), 1, fp) != 1)
				    goto rderr_exit;
			 if (mgetwl(bfh + BFH_WTYPE) == BMP_SIGNATURE) break;
			 if (i == 0) return err_not_a_bmp;
			 if (fread(bfh, (128 - FILEHED_SIZE - BIHSIZE_SIZE), 1, fp) != 1)
				    goto rderr_exit;
	   }
	   offbits = mgetdwl(bfh + BFH_DOFFBITS);
	   bihsize = mgetdwl(bfh + BFH_DBIHSIZE);
	   skip    = offbits - bihsize - FILEHED_SIZE;
	   if (bihsize < COREHED_SIZE || bihsize > BMPV5HED_SIZE ||
		  offbits < (bihsize + FILEHED_SIZE)) return err_invalid_hed;

	   if (fread((bih + BIHSIZE_SIZE), (bihsize - BIHSIZE_SIZE), 1, fp) != 1)
			 goto rderr_exit;

	   if (bihsize == COREHED_SIZE) {	/* OS/2-Type BMP */
			 img->width    = mgetwl(bih + BCH_WWIDTH);
			 img->height   = mgetwl(bih + BCH_WHEIGHT);
			 img->pixdepth = mgetwl(bih + BCH_WBITCOUNT);
			 img->topdown  = FALSE;
			 compression   = BI_RGB;
			 palsize       = RGBTRIPLE_SIZE;
	   } else {						/* Windows-Type BMP */
			 img->width    = mgetdwl(bih + BIH_LWIDTH);
			 img->height   = mgetdwl(bih + BIH_LHEIGHT);
			 img->pixdepth = mgetwl(bih + BIH_WBITCOUNT);
			 img->topdown  = FALSE;
			 compression   = mgetdwl(bih + BIH_DCOMPRESSION);
			 palsize       = RGBQUAD_SIZE;
			 if (img->height < 0) {
				    img->height  = -img->height;
				    img->topdown = TRUE;	/* top-down BMP */
			 }
	   }
	   if (img->width <= 0)  return err_width_zero;
	   if (img->height <= 0) return err_height_zero;
	   if (compression != BI_RGB) return err_compressed;
	   if (img->pixdepth != 1 && img->pixdepth != 4 &&
		  img->pixdepth != 8 && img->pixdepth != 24) {
			 if (img->pixdepth == 16 || img->pixdepth == 32) return err_16or32bpp;
			 else return err_invalid_bpp;
	   }
	   if (img->pixdepth != 24) {
			 img->palnum = skip / palsize;
			 skip        = skip % palsize;
	   } else {
			 img->palnum = 0;
	   }
	   if (!imgalloc(img)) return err_outofmemory;

	   /* ------------------------------------------------------ */

	   for (pal = img->palette, i = img->palnum; i > 0; i--, pal++) {
			 if (fread(rgbq, palsize, 1, fp) != 1) goto rderr_exit2;
			 pal->red   = rgbq[RGBQ_RED];
			 pal->green = rgbq[RGBQ_GREEN];
			 pal->blue  = rgbq[RGBQ_BLUE];
	   }
	   for (i = skip; i > 0; i--)
			 if (fgetc(fp) == EOF) goto rderr_exit2;

	   /* ------------------------------------------------------ */

	   if (fread(img->bmpbits, img->imgbytes, 1, fp) != 1) goto rderr_exit2;

	   /* ------------------------------------------------------ */

	   // set_status("Read OK %s", basname(fn));

	   return NULL;	/* success */

	   /* error */
rderr_exit2:
	   imgfree(img);
rderr_exit:
	   return ferror(fp) ? err_readerr : err_readeof;
}


static unsigned long 
mgetdwl(void *ptr)
{
	   unsigned char *p = (unsigned char*)ptr;
	   unsigned long v;

	   v = ((unsigned long)p[0]      ) + ((unsigned long)p[1] <<  8) +
			 ((unsigned long)p[2] << 16) + ((unsigned long)p[3] << 24);

	   return v;
}


static unsigned int 
mgetwl(void *ptr)
{
	   unsigned char *p = (unsigned char*)ptr;

	   return ((unsigned int)p[0]) + ((unsigned int)p[1] << 8);
}


/* -----------------------------------------------------------------------
**		PNG 
*/

static int 
write_png(char *fn, FILE *fp, IMAGE *img)
{
	   png_structp png_ptr;
	   png_infop info_ptr;
	   int bit_depth;
	   int color_type;
	   int interlace_type;


	   /* ------------------------------------------------------ */

	   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, fn,
					     png_my_error, png_my_warning);
	   if (png_ptr == NULL) {
	   		fprintf (stderr, err_outofmemory, fn);
			 return 0;
	   }
	   info_ptr = png_create_info_struct(png_ptr);
	   if (info_ptr == NULL) {
			 png_destroy_write_struct(&png_ptr, NULL);
			 fprintf (stderr, err_outofmemory, fn);
			 return 0;
	   }
	   if (setjmp(png_ptr->jmpbuf)) {
			 /* If we get here, we had a problem reading the file */
			 png_destroy_write_struct(&png_ptr, &info_ptr);
			 return 0;
	   }
	   png_init_io(png_ptr, fp);
	   png_set_compression_level(png_ptr, complevel);
	   if (filters != 0)
			 png_set_filter(png_ptr, PNG_FILTER_TYPE_BASE, filters);

	   /* ------------------------------------------------------ */

	   if (img->pixdepth == 24) {
			 bit_depth  = 8;
			 color_type = PNG_COLOR_TYPE_RGB;
			 png_set_compression_mem_level(png_ptr, MAX_MEM_LEVEL);
	   } else {
			 bit_depth  = img->pixdepth;
			 color_type = PNG_COLOR_TYPE_PALETTE;
			 png_set_PLTE(png_ptr, info_ptr, img->palette, img->palnum);
	   }
	   interlace_type = (interlace) ? PNG_INTERLACE_ADAM7 : PNG_INTERLACE_NONE;

	   png_set_IHDR(png_ptr, info_ptr, img->width, img->height, bit_depth,
				 color_type, interlace_type, PNG_COMPRESSION_TYPE_DEFAULT,
				 PNG_FILTER_TYPE_DEFAULT);

	   png_write_info(png_ptr, info_ptr);

	   /* ------------------------------------------------------ */

	   if (img->pixdepth == 24) png_set_bgr(png_ptr);

	   /* ------------------------------------------------------ */

	   png_set_write_status_fn(png_ptr, row_callback);
	   init_progress_meter(png_ptr, img->width, img->height);

	   png_write_image(png_ptr, img->rowptr);

	   png_write_end(png_ptr, info_ptr);
	   png_destroy_write_struct(&png_ptr, &info_ptr);

	   /* ------------------------------------------------------ */
	   
	   return 1;
}


/* -----------------------------------------------------------------------
 */



#if (PNG_LIBPNG_VER >= 10007)
/*
**	dummy - see png_reset_zstream() in png.c
*/
int inflateReset(z_streamp z)
{
	   return Z_OK;
}
#endif
