/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*
**  bmp2png --- conversion from (Windows or OS/2 style) BMP to PNG
**  png2bmp --- conversion from PNG to (Windows style) BMP
**
**  Copyright (C) 1999-2002 MIYASAKA Masaru <alkaid@coral.ocn.ne.jp>
**
**  Permission to use, copy, modify, and distribute this software and
**  its documentation for any purpose and without fee is hereby granted,
**  provided that the above copyright notice appear in all copies and
**  that both that copyright notice and this permission notice appear
**  in supporting documentation. This software is provided "as is"
**  without express or implied warranty.
**
**  NOTE: Comments are partly written in Japanese. Sorry.
*/

#ifndef BMPHED_H
#define BMPHED_H

/* BMP file signature */
#define BMP_SIGNATURE           0x4d42
#define BMP_SIG_BYTES           2

/* BITMAPFILEHEADER */
#define BFH_WTYPE                       0               /* WORD   bfType;           */
#define BFH_DSIZE                       2               /* DWORD  bfSize;           */
#define BFH_WRESERVED1          6               /* WORD   bfReserved1;      */
#define BFH_WRESERVED2          8               /* WORD   bfReserved2;      */
#define BFH_DOFFBITS            10              /* DWORD  bfOffBits;        */
#define BFH_DBIHSIZE            14              /* DWORD  biSize;           */
#define FILEHED_SIZE            14              /* sizeof(BITMAPFILEHEADER) */
#define BIHSIZE_SIZE            4               /* sizeof(biSize)           */

/* BITMAPINFOHEADER */
#define BIH_DSIZE                       0               /* DWORD  biSize;           */
#define BIH_LWIDTH                      4               /* LONG   biWidth;          */
#define BIH_LHEIGHT                     8               /* LONG   biHeight;         */
#define BIH_WPLANES                     12              /* WORD   biPlanes;         */
#define BIH_WBITCOUNT           14              /* WORD   biBitCount;       */
#define BIH_DCOMPRESSION        16              /* DWORD  biCompression;    */
#define BIH_DSIZEIMAGE          20              /* DWORD  biSizeImage;      */
#define BIH_LXPELSPERMETER      24              /* LONG   biXPelsPerMeter;  */
#define BIH_LYPELSPERMETER      28              /* LONG   biYPelsPerMeter;  */
#define BIH_DCLRUSED            32              /* DWORD  biClrUsed;        */
#define BIH_DCLRIMPORANT        36              /* DWORD  biClrImportant;   */
#define INFOHED_SIZE            40              /* sizeof(BITMAPINFOHEADER) */
#define BMPV4HED_SIZE           108             /* sizeof(BITMAPV4HEADER)   */
#define BMPV5HED_SIZE           124             /* sizeof(BITMAPV5HEADER)   */

/* BITMAPCOREHEADER */
#define BCH_DSIZE                       0               /* DWORD  bcSize;           */
#define BCH_WWIDTH                      4               /* WORD   bcWidth;          */
#define BCH_WHEIGHT                     6               /* WORD   bcHeight;         */
#define BCH_WPLANES                     8               /* WORD   bcPlanes;         */
#define BCH_WBITCOUNT           10              /* WORD   bcBitCount;       */
#define COREHED_SIZE            12              /* sizeof(BITMAPCOREHEADER) */

/* RGBQUAD */
#define RGBQ_BLUE                       0               /* BYTE   rgbBlue;     */
#define RGBQ_GREEN                      1               /* BYTE   rgbGreen;    */
#define RGBQ_RED                        2               /* BYTE   rgbRed;      */
#define RGBQ_RESERVED           3               /* BYTE   rgbReserved; */
#define RGBQUAD_SIZE            4               /* sizeof(RGBQUAD)     */

/* RGBTRIPLE */
#define RGBT_BLUE                       0               /* BYTE   rgbtBlue;    */
#define RGBT_GREEN                      1               /* BYTE   rgbtGreen;   */
#define RGBT_RED                        2               /* BYTE   rgbtRed;     */
#define RGBTRIPLE_SIZE          3               /* sizeof(RGBTRIPLE)   */

/* Constants for the biCompression field */
#ifndef BI_RGB
#define BI_RGB                          0L              /* Uncompressed format       */
#define BI_RLE8                         1L              /* RLE format (8 bits/pixel) */
#define BI_RLE4                         2L              /* RLE format (4 bits/pixel) */
#define BI_BITFIELDS            3L              /* Bitfield format           */
#define BI_JPEG                         4L              /* JPEG format ?             */
#define BI_PNG                          5L              /* PNG format ??             */
#endif

#endif /* BMPHED_H */




#ifndef COMMON_H
#define COMMON_H

#if defined(__RSXNT__) && defined(__CRTRSXNT__)
# include <crtrsxnt.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

/* for stat() */
#include <sys/types.h>
#include <sys/stat.h>

/* for utime() */
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__LCC__)
# include <sys/utime.h>
# if defined(__LCC__)
int utime(const char *, struct _utimbuf *);
# endif
#else
# include <utime.h>
#endif
/* for isatty() */
#if defined(_MSC_VER) || defined(__BORLANDC__) || defined(__MINGW32__) || \
    defined(__LCC__)
# include <io.h>
#else
# include <unistd.h>
#endif
/* for mkdir() */
#if defined(_MSC_VER) || defined(__BORLANDC__) || defined(__MINGW32__) || \
    defined(__LCC__)
# include <direct.h>
# define MKDIR(d,m) mkdir(d)
#else
# if defined(__GO32__) && !defined(__DJGPP__)   /* DJGPP v.1 */
#  include <osfcn.h>
# else
#  include <sys/stat.h>
# endif
# define MKDIR(d,m) mkdir(d,m)
#endif

#if !defined(USE_FDOPEN) && !defined(USE_SETMODE)
# if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__RSXNT__) || \
     defined(_MSC_VER) || defined(__BORLANDC__) || defined(__LCC__) || \
     defined(__DJGPP__) || defined(__GO32__)
#  define USE_SETMODE
# endif
# if 0 /* defined(__YOUR_COMPLIER_MACRO__) */
#  define USE_FDOPEN
# endif
#endif
/* for setmode() */
#ifdef USE_SETMODE
# include <io.h>
# include <fcntl.h>
#endif

#include "png.h"

#include <setjmp.h>

#if (defined(_WIN32) || defined(__WIN32__)) && !defined(WIN32)
# define WIN32
#endif
#if defined(__MSDOS__) && !defined(MSDOS)
# define MSDOS
#endif
#if defined(__CYGWIN__)
# undef WIN32
# undef MSDOS
#endif

#if defined(WIN32) || defined(__DJGPP__)
# define WIN32_LFN              /* Win32-style long filename */
#endif

#if defined(WIN32) || defined(MSDOS)
# define PATHDELIM              '\\'
# define DRIVESUFFIX    ':'
# define IsPathDelim(c) ((c)==PATHDELIM || (c)=='/')
# define IsOptChar(c)   ((c)=='-' || (c)=='/')
# ifdef JAPANESE
#  define IsDBCSLead(c) ((0x81<=(c) && (c)<=0x9F) || (0xE0<=(c) && (c)<=0xFC))
# else
#  define IsDBCSLead(c) (0)
# endif
#else   /* UNIX */
# define PATHDELIM              '/'
# define IsPathDelim(c) ((c)==PATHDELIM)
# define IsOptChar(c)   ((c)=='-')
# define IsDBCSLead(c)  (0)
#endif

typedef char           CHAR;
typedef unsigned char  BYTE;
typedef short          SHORT;
typedef unsigned short WORD;
typedef int            INT;
typedef unsigned int   UINT;
typedef long           LONG;
typedef unsigned long  DWORD;
typedef enum { FALSE = 0, TRUE = 1 } BOOL;

typedef png_color PALETTE;
typedef struct tagIMAGE {
	   LONG    width;
	   LONG    height;
	   UINT    pixdepth;
	   UINT    palnum;
	   BOOL    topdown;
	   /* ----------- */
	   DWORD   rowbytes;
	   DWORD   imgbytes;
	   PALETTE *palette;
	   BYTE    **rowptr;
	   BYTE    *bmpbits;
} IMAGE;

extern int quietmode;
extern int errorlog;
extern char errlogfile[];

// void xxprintf(const char *, ...);
// void set_status(const char *, ...);
// void feed_line(void);
void init_progress_meter(png_structp, png_uint_32, png_uint_32);
void row_callback(png_structp, png_uint_32, int);
void png_my_error(png_structp, png_const_charp);
void png_my_warning(png_structp, png_const_charp);
BOOL imgalloc(IMAGE *);
void imgfree(IMAGE *);
int parsearg(int *, char **, int, char **, char *);
char **envargv(int *, char ***, const char *);
int tokenize(char *, const char *);
int makedir(const char *);
int renbak(const char *);
int cpyftime(const char *, const char *);
char *suffix(const char *);
char *basname(const char *);
char *addslash(char *);
char *delslash(char *);
char *path_skiproot(const char *);
char *path_nextslash(const char *);
#ifdef WIN32_LFN
int is_dos_filename(const char *);
#endif



int read_file(char *, IMAGE *);
int write_file(char *, IMAGE *);

#endif  /* COMMON_H */


