/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "Drawable.h"

#include <string>
#include <iostream>

using namespace std;
using namespace slideshow;


Drawable::Drawable() :
	DrawableEvents()
{
	x = y = 0;
	name        = NULL;
	surface     = NULL;
	scroll      = NULL;
	need_redraw = true;
	stage       = 0;
}


Drawable::~Drawable()
{
	if (surface) {
		SDL_FreeSurface(surface);
	}
}


void 
Drawable::Draw (SDL_Surface *screen)
{
	SDL_Rect dest;

	if (surface) {
		if ((scroll) && (scroll->Finished() == false)) {
			scroll->Move();
		}
			 
		dest.x = x;
		dest.y = y;
		dest.w = surface->w;
		dest.h = surface->h;
		   
		SDL_BlitSurface (surface, NULL, screen, &dest);
		need_redraw = false;
	}
}


Uint16
Drawable::Width ()
{
	if (surface) {
		return surface->w;
	}

	return 0;
}


Uint16
Drawable::Height ()
{
	if (surface) {
		return surface->h;
	}

	return 0;
}


void
Drawable::AlignLeft  (void)
{
	x = 0;
}


void
Drawable::AlignRight (int screen_width)
{
	x = screen_width - Width();
}


void
Drawable::AlignUp (void)
{
	y = 0;
}


void
Drawable::AlignDown  (int screen_height)
{
	y = screen_height - Height();
}


void
Drawable::AlignCenter (int screen_width, int screen_height)
{
	AlignHorizontalCenter (screen_width);
	AlignVerticalCenter (screen_height);
}


void
Drawable::AlignHorizontalCenter(int screen_width)
{
	x = (screen_width - Width()) / 2;
}


void
Drawable::AlignVerticalCenter(int screen_height)
{
	y = (screen_height - Height()) / 2;
}


void
Drawable::SetScroll (int direction, int stop_at, int another_pos, int pixels_per_second)
{
	if (scroll) {
		delete scroll;
	}

	scroll = new Scroll ((int)Width(), (int)Height(), &x, &y, direction, stop_at, another_pos, pixels_per_second);
}


bool
Drawable::NeedRedraw (void)
{
	bool tmp = need_redraw;
	
	if (scroll) {
		tmp |= (!scroll->Finished());
	}
	
	return tmp;
}


void
Drawable::SayCheese (void)
{	
	if (scroll) {
		scroll->MakeFinish();
	}
}

/*
string
Drawable::ToPython (void)
{
	cerr << "ToPython not implemented" << endl;
	return string();
}


string
Drawable::ToXML (void)
{
	cerr << "ToXML not implemented" << endl;
	return string();
}

string
Drawable::ToCPP (void)
{
	cerr << "ToCPP not implemented" << endl;
	return string();
}
*/

