/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __Drawable__
#define __Drawable__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <SDL.h>

#include <string>
using std::string;

#include "Macros.h"
#include "Scroll.h"

class Namer;
class Effect;

namespace slideshow {
	   class DrawableEvents {
	   public:
			 virtual void On_JoiningPage (void) {};
			 virtual void On_LeavingPage (void) {};
	   };


	   class Drawable : public DrawableEvents {
	   public:
			 SET_CLASS(Drawable);
	   
			 Drawable();
			 virtual ~Drawable();

			 int x;
			 int y;
			 int stage;

			 virtual void Draw (SDL_Surface *screen);
			 virtual bool NeedRedraw (void);
	   
			 void AlignLeft  (void);
			 void AlignRight (int screen_width  = DEFAULT_WIDTH);
			 void AlignUp    (void);
			 void AlignDown  (int screen_height = DEFAULT_HEIGHT);

			 void AlignCenter(int screen_width  = DEFAULT_WIDTH,
						   int screen_height = DEFAULT_HEIGHT);
			 void AlignVerticalCenter  (int screen_height = DEFAULT_HEIGHT); 
			 void AlignHorizontalCenter(int screen_width  = DEFAULT_WIDTH);

	   
			 void SetScroll (int direction, int stop_at, int another_pos=-1, int pixels_per_second = 300);
	   
			 virtual Uint16 Width (void);
			 virtual Uint16 Height (void);

			 virtual void SayCheese (void);
	
			 virtual string ToPython (void) const = 0;
			 virtual string ToXML (void) const = 0;
			 virtual string ToCPP (void) const = 0;
		 
	   protected:
			 string      *name;
			 SDL_Surface *surface;
			 Scroll      *scroll;
			 bool         need_redraw;
	   
			 friend class Namer;
			 friend class Effect;
	   };
}


#endif // __Drawable__
