/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "EffectDarkness.h"
#include "Macros.h"
#include <assert.h>


EffectDarkness::EffectDarkness (Drawable *parent, int darkmax_) :
	Effect (parent)
{
	   dark = 0;
	   darkmax = darkmax_;
}


EffectDarkness::~EffectDarkness()
{
}


void
EffectDarkness::SayCheese (void)
{
	dark = darkmax;
}


void
EffectDarkness::Draw (SDL_Surface *screen)
{
	if (!parent_surface) return;

	Uint32 color;
	Uint8  r,g,b;
	for (int i=0; i < parent_surface->w; i++) {
		for (int j=0; j < parent_surface->h; j++) {
			GET_PIXEL(parent_surface,i,j,color);

			COLOR_TO_RGB(color,r,g,b);
			
			r = (r-dark > 0) ? r-dark : 0;
			g = (g-dark > 0) ? g-dark : 0;
			b = (b-dark > 0) ? b-dark : 0;

			PUT_PIXEL(screen,i,j,r,g,b);
		}
	}
	   
	if (dark < darkmax) {
		dark ++;
	}
}


void
EffectDarkness::ProcessEffect (void)
{
}


string
EffectDarkness::ToPython (void) const
{
	return "";
}


string
EffectDarkness::ToXML (void) const
{
	return "";
}


string
EffectDarkness::ToCPP (void) const
{
	return "";
}
