/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Dave Ashley <dash@xdr.com>
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*
 * This class is based on the "Fireworks demo" written by Dave Ashley <dash@xdr.com>
 */

#include <stdlib.h>
#include <string.h>

#include "Macros.h"
#include "Fires.h"


#define RATE 1
#define MAXBLOBS 512
#define BLOBFRAC 6
#define BLOBGRAVITY 5
#define THRESHOLD 20
#define SMALLSIZE 3
#define BIGSIZE 6


char sqrttab[]={
	   0,1,1,1,2,2,2,2,2,3,3,3,3,3,3,3,
	   4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,
	   5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,
	   6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	   8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
	   8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
	   9,9,9,9,10,10,10,10,10,10,10,10,10,10,10,10,
	   10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,11,
	   11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,
	   12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
	   12,12,12,12,12,12,12,12,12,13,13,13,13,13,13,13,
	   13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
	   13,13,13,13,14,14,14,14,14,14,14,14,14,14,14,14,
	   14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
	   14,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
	   15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
};



Fires::Fires (int screen_width, int screen_height, int whichmap, bool trans) :
	slideshow::Drawable()
{
	   XSIZE = screen_width;
	   YSIZE = screen_height;

        Uint32 rmask, gmask, bmask, amask;

	   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
	   rmask = 0xff000000;
	   gmask = 0x00ff0000;
	   bmask = 0x0000ff00;
	   amask = 0x000000ff;
	   #else
	   rmask = 0x000000ff;
	   gmask = 0x0000ff00;
	   bmask = 0x00ff0000;
	   amask = 0xff000000;
	   #endif

	   surface = SDL_CreateRGBSurface (SDL_SWSURFACE, screen_width, screen_height, 8,
					   rmask, gmask, bmask, amask);

	   if (trans) {
		   SetTransparent(trans);
	   }
	   
	   dest.x = 0;
	   dest.y = 0;
	   dest.w = surface->w;
	   dest.h = surface->h;

	   
	   sizes[0] = 2;
	   sizes[1] = 3;
	   sizes[2] = 4;
	   sizes[3] = 5;
	   sizes[4] = 8;
	   sizes[5] = 5;
	   sizes[6] = 4;
	   sizes[7] = 3;
	   
	   frames = now = 0;
	   flash = 0;
	   explodenum = 100;
	   
	   vmem1 = NULL;
	   vmem2 = (unsigned char *) malloc(XSIZE*YSIZE);
	   mul640= (unsigned char **) malloc(YSIZE*sizeof(char *));
	   remap = (char *) malloc(16384);
	   remap2= (char *) malloc(16384);
	   blobs = (blob *) malloc(MAXBLOBS*sizeof(struct blob));

        freeblobs=activeblobs=0;
	   for(int i=0; i<MAXBLOBS; i++)
	   {
			 blobs[i].blobnext=freeblobs;
			 freeblobs=blobs+i;
	   }
	   
	   normal(remap);
	   bright(remap2);

	   loadcolors(whichmap);
}


Fires::~Fires ()
{
	   if (vmem1) free(vmem1);
	   if (vmem2) free(vmem2);
	   if (mul640) free(mul640);
	   if (remap) free(remap);
	   if (remap2) free(remap2);
	   if (blobs) free(blobs);
}


void
Fires::SetTransparent (bool value)
{
	if (value) {
		SDL_SetColorKey(surface, SDL_SRCCOLORKEY, SDL_MapRGB(surface->format, 0,0,0));
	} else {
		// TODO !!!!
		SS_ASSERT (0);
	}
}


void
Fires::Draw (SDL_Surface *screen)
{
	   frames++;
	   if ( vmem1 != (unsigned char *)surface->pixels )
	   {
			 p1 = vmem1 = (unsigned char *) surface->pixels;
			 for (int i=0;i<YSIZE;i++)
			 {
				    mul640[i] = i * surface->pitch+vmem1;
				    memset(p1,0,XSIZE);
				    p1 += surface->pitch;
			 }
	   }
	   

	   now++;
	   if(!flash)
	   {
			 if(explodenum>96 && explodenum<160 && !(rand()&511))
				    flash=60;
	   } else --flash;
	   explodenum=(now>>4)+1;if(explodenum==320) now=0;
	   if(explodenum>256) explodenum=256;
	   if(!(rand()&31))
			 addblob();
	   moveblobs();
	   putblobs();
	   p1=vmem1;
	   p2=vmem2;
	   k=surface->pitch;
	   for(int i=0; i<YSIZE; i++)
	   {
			 memcpy(p2,p1,XSIZE);
			 p2+=XSIZE;
			 p1+=k;
	   }
	   fire(vmem2,vmem1,k,flash ? remap2 :remap);	   


	   SDL_BlitSurface (surface, NULL, screen, &dest);
}


void
Fires::fire (unsigned char *p1,unsigned char *p2,int pitch,char *map)
{
	   int x,y;
	   unsigned char *p3, *p4;

	   for(y=2;y<YSIZE;y++)
	   {
			 for(x=0;x<XSIZE;x++)
			 {
				    p3 = p1+y*XSIZE+x;
				    p4 = p2+y*pitch+x;
				    *p4=map[*p3+p3[-XSIZE]+p3[-XSIZE-1]+p3[-XSIZE+1]+p3[-1]+p3[1]+p3[-XSIZE-XSIZE-1]+p3[-XSIZE-XSIZE]+p3[-XSIZE-XSIZE+1]];
			 }
	   }
}


void
Fires::disk(int x, int y, int rad)
{
	   unsigned char *p;
	   int i,j,k,aj;
	   int rad2=rad*rad;
	   int w;


	   for(j=-rad;j<=rad;j++)
	   {
			 w=sqrttab[rad2-j*j];
			 aj=ABS(j)<<2;
			 if(w)
			 {
				    p=mul640[y+j]+x-w;
				    k=w+w+1;
				    i=-w;
				    while(k--) {*p++=255-(ABS(i)<<2)-aj;i++;}
			 }
	   }
}



void
Fires::addblob(void)
{
	   int dx,dy;
	   struct blob *ablob;

	   if(!freeblobs) return;
	   dx=(rand()&255)-128;
	   dy=(rand()%200)+340;
	   ablob=freeblobs;
	   freeblobs=freeblobs->blobnext;
	   ablob->bloblife=(rand()&511)+256;
	   ablob->blobdx=dx;
	   ablob->blobdy=dy;
	   ablob->blobx=(256+(rand()&127))<<BLOBFRAC;
	   ablob->bloby=2<<BLOBFRAC;
	   ablob->blobnext=activeblobs;
	   ablob->blobsize=BIGSIZE;
	   activeblobs=ablob;
}


void
Fires::moveblobs(void)
{
	   struct blob **lastblob,*ablob;
	   int x,y;

	   lastblob=&activeblobs;
	   while ((ablob=*lastblob))
	   {
			 x=ablob->blobx>>BLOBFRAC;
			 y=ablob->bloby>>BLOBFRAC;
			 if(!--ablob->bloblife || y<0 || x<10 || x>XSIZE-10)
			 {
				    *lastblob=ablob->blobnext;
				    ablob->blobnext=freeblobs;
				    freeblobs=ablob;
				    continue;
			 }
			 ablob->blobx+=ablob->blobdx;
			 ablob->bloby+=ablob->blobdy;
			 ablob->blobdy-=BLOBGRAVITY;
			 lastblob=&ablob->blobnext;
	   }
}


void
Fires::putblobs(void)
{
	   struct blob *ablob,*ablob2,*temp;
	   int x,y,dy;
	   int i,size;
	   long x2,y2,vel;

	   ablob=activeblobs;
	   activeblobs=0;
	   while(ablob)
	   {
			 dy=ablob->blobdy;
			 if(ablob->blobsize!=SMALLSIZE && (dy>-THRESHOLD && dy<THRESHOLD && !(rand()&7) || (rand()&127)==63))
			 {
				    i=explodenum;
				    while(i-- && freeblobs)
				    {
						  ablob2=freeblobs;
						  freeblobs=freeblobs->blobnext;
						  ablob2->blobx=ablob->blobx;
						  ablob2->bloby=ablob->bloby;
						  for(;;)
						  {
								x2=(rand()&511)-256;
								y2=(rand()&511)-256;
								vel=x2*x2+y2*y2;
								if(vel>0x3000 && vel<0x10000L) break;
						  }
						  ablob2->blobdx=ablob->blobdx+x2;
						  ablob2->blobdy=ablob->blobdy+y2;
						  ablob2->bloblife=16+(rand()&31);
						  ablob2->blobsize=SMALLSIZE;
						  ablob2->blobnext=activeblobs;
						  activeblobs=ablob2;
						  ablob->bloblife=1;
				    }
			 }
			 x=ablob->blobx>>BLOBFRAC;
			 y=ablob->bloby>>BLOBFRAC;
			 size=ablob->blobsize;
			 if(size==BIGSIZE && ablob->blobdy>0 && ablob->blobdy<200)
				    size=sizes[ablob->bloblife&7];
			 if(x>10 && x<XSIZE-10 && y>10 && y<YSIZE-10)
				    disk(x,YSIZE-1-y,size);
			 temp=ablob;
			 ablob=ablob->blobnext;
			 temp->blobnext=activeblobs;
			 activeblobs=temp;
	   }
}


void
Fires::normal(char *map)
{
	   int i,j;
	   for(i=0;i<8192;i++)
	   {
			 j=i/9;
			 map[i]=j<256 ? (j>=RATE ? j-RATE : 0) : 255;
	   }
}


void
Fires::bright(char *map)
{
	   int i;
	   for(i=0;i<8192;i++) map[i]=i>>3<255 ? (i>>3) : 255;
}


void
Fires::updatemap(void)
{
	   SDL_SetColors(surface, themap, 0, 256);
}


void
Fires::loadcolor(int n,int r,int g,int b)
{
	   themap[n].r=r<<2;
	   themap[n].g=g<<2;
	   themap[n].b=b<<2;
}


void
Fires::loadcolors(unsigned int which)
{
	   int i,j;
	   int r,g,b;

	   which%=11;
	   for(i=0;i<256;i++)
	   {
			 switch(which)
			 {
			 case 0:
				    if(i<64) loadcolor(i,0,0,0);
				    else if(i<128)  loadcolor(i,i-64,0,0);
				    else if(i<192) loadcolor(i,63,i-128,0);
				    else loadcolor(i,63,63,i-192);
				    break;
			 case 1:
				    if(i<64) loadcolor(i,0,0,0);
				    else if(i<128)  loadcolor(i,0,0,i-64);
				    else loadcolor(i,(i-128)>>1,(i-128)>>1,63);
				    break;
			 case 2:
				    loadcolor(i,i>>2,i>>2,i>>2);
				    break;
			 case 3:
				    r=rand()&0x3f;
				    g=rand()&0x3f;
				    b=rand()&0x3f;
				    loadcolor(i,r*i>>8,g*i>>8,b*i>>8);
				    break;
			 case 4:
				    loadcolor(i,i>>2,0,0);
				    break;
			 case 5:
				    loadcolor(i,0,i>>2,0);
				    break;
			 case 6:
				    loadcolor(i,0,0,i>>2);
				    break;
			 case 7:
				    j=i&15;
				    if(i&16) j=15-j;
				    j=(i>>2)*j/16;
				    loadcolor(i,j,j,j);
				    break;
			 case 8:
				    j=0;
				    if(i>8 && i<128) j=63;
				    loadcolor(i,j,j,j);
				    break;
			 case 9:
				    j=31-(i&31)<<1;
				    r=i&32 ? j : 0;
				    g=i&64 ? j : 0;
				    b=i&128 ? j : 0;
				    loadcolor(i,r,g,b);
				    break;
			 case 10:
				    j=(i&15)<<2;
				    if(i&16) j=63-j;
				    r=i&32 ? j : 0;
				    g=i&64 ? j : 0;
				    b=i&128 ? j : 0;
				    loadcolor(i,r,g,b);
				    break;
			 }
	   }
	   updatemap();
}


string
Fires::ToPython (void) const
{
	return "";
}


string
Fires::ToXML (void) const
{
	return "";
}


string
Fires::ToCPP (void) const
{
	return "";
}
