/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#ifndef __FontVault__
#define __FontVault__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <SDL.h>
#include <SDL_ttf.h>

#ifdef HAVE_GNU_CXX_HASHMAP
#include <ext/hash_map>
#else
#include <hash_map.h>
#endif

#include <string>

#include "Macros.h"


struct ures_eqstr {
	   bool operator()(const std::string& s1, const std::string& s2) const {
			 return s1 == s2;
	   }
};

struct ures_hasher {
	   size_t operator () (const std::string& s) const {
			 unsigned long __h = 0;
			 std::string::const_iterator itr = s.begin();
			 while(itr != s.end())
				    __h = 5*__h + *itr++;

			 return size_t(__h);
	   }
};

#if (HASHMAP_NAMESPACE == __gnu_cxx)
       typedef __gnu_cxx::hash_map<std::string, TTF_Font*, ures_hasher, ures_eqstr> Fonts_HashMap;
#else
       typedef std::hash_map<std::string, TTF_Font*, ures_hasher, ures_eqstr> Fonts_HashMap;
#endif



class FontVault {
public:
	   FontVault();
	   ~FontVault();

	   void      Load       (const char *name, const char *filename, int size);
	   TTF_Font *Get        (const char *fontname);
	   TTF_Font *operator[] (const char *fontname) { return Get(fontname); }
	   
	   TTF_Font *GetFirstOne (void);

	   
protected:
	   Fonts_HashMap fonts;
};



#endif // __FontVault__
