/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include <assert.h>
#include "Gradient.h"
#include "Macros.h"


Gradient::Gradient (int c1, int c2, SlideShowGradient type_) :
	   slideshow::Drawable()
{
	   color1 = c1;
	   color2 = c2;
	   type   = type_;
}


void
Gradient::DrawVertical (SDL_Surface *screen,
			Uint8 r1, Uint8 g1, Uint8 b1,
			Uint8 r2, Uint8 g2, Uint8 b2)
{
	   Uint8 r, g, b;	   

	   for (int j=0; j<screen->h; j++) {
			 if (r1 > r2) {
				    r = (r1 - r2) * j / (float)screen->h;
			 } else {
				    r = (r2 - r1) * j / (float)screen->h;
			 }
			 
			 if (g1 > g2) {
				    g = (g1 - g2) * j / (float)screen->h;
			 } else {
				    g = (g2 - g1) * j / (float)screen->h;
			 }

			 if (b1 > b2) {
				    b = (b1 - b2) * j / (float)screen->h;
			 } else {
				    b = (b2 - b1) * j / (float)screen->h;
			 }

			 for (int i=0; i<screen->w; i++) {
				 PUT_PIXEL(screen,i,j,r,g,b);
			 }
	   }
}


void
Gradient::DrawHorizontal (SDL_Surface *screen,
			Uint8 r1, Uint8 g1, Uint8 b1,
			Uint8 r2, Uint8 g2, Uint8 b2)
{
	   Uint8 r, g, b;	   

	   for (int i=0; i<screen->w; i++) {
			 if (r1 > r2) {
				    r = (r1 - r2) * i / (float)screen->w;
			 } else {
				    r = (r2 - r1) * i / (float)screen->w;
			 }
			 
			 if (g1 > g2) {
				    g = (g1 - g2) * i / (float)screen->w;
			 } else {
				    g = (g2 - g1) * i / (float)screen->w;
			 }

			 if (b1 > b2) {
				    b = (b1 - b2) * i / (float)screen->w;
			 } else {
				    b = (b2 - b1) * i / (float)screen->w;
			 }

			 for (int j=0; j<screen->h; j++) {
				 PUT_PIXEL(screen,i,j,r,g,b);
			 }
	   }
}


void
Gradient::Draw (SDL_Surface *screen)
{
	   Uint8 r1, g1, b1;	   
	   Uint8 r2, g2, b2;	   

	   COLOR_TO_RGB(color1,r1,g1,b1);
	   COLOR_TO_RGB(color2,r2,g2,b2);

	   if (type == GRADIENT_VERTICAL) {
		   DrawVertical (screen, r1,g1,b1, r2,g2,b2);
	   } else if (type == GRADIENT_HORIZONTAL) {
		   DrawHorizontal (screen, r1,g1,b1, r2,g2,b2);
	   }
	   
	   need_redraw = false;
}


string
Gradient::ToPython (void) const
{
	return "";
}


string
Gradient::ToXML (void) const
{
	return "";
}


string
Gradient::ToCPP (void) const
{
	return "";
}
