/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "GtkPresentation.h"
#include "gtksdl.h"
#include "HtmlExporter.h"


//
#include <iostream>
//


gboolean
delete_event_cb (GtkWidget *widget, GdkEventAny *event)
{
        gtk_main_quit();
	return TRUE ;
}


gboolean
idle_loop (gpointer data)
{
	bool ret;
	GtkPresentation *self = static_cast<GtkPresentation*>(data);

	g_assert (self && self->presentation);

	if (!self->presentation->IsEmpty()) {
		ret = !self->presentation->DoMainloopIteration();
	}

	return ret;
}


gboolean
button_quit_clicked (GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	gtk_main_quit();
	return TRUE;
}

/*
gboolean
gtk_sdl_redraw (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	cout << "PRUEBA gtk_sdl_redraw" << endl;
	return FALSE;
}
*/

gboolean
menu_load_file_ok_clicked (GtkWidget *widget, gpointer data)
{
 	GtkWidget *fs = GTK_WIDGET(data);
	GtkPresentation *self = static_cast<GtkPresentation*>(gtk_object_get_data(GTK_OBJECT(fs), "self"));

	char *filename = (char *)gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	self->Load(filename);

	gtk_widget_destroy(fs);
}


gboolean
menu_load_file (gpointer data, guint callback_action, GtkWidget *widget)
{	
	GtkPresentation *self = static_cast<GtkPresentation*>(data);
	GtkWidget *fs = gtk_file_selection_new ("Load");
	gtk_object_set_data(GTK_OBJECT(fs), "self", self);
	
	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
			  "clicked",
			  G_CALLBACK (menu_load_file_ok_clicked), fs);

	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
				  "clicked",
				  G_CALLBACK (gtk_widget_destroy), fs);

	gtk_widget_show (fs);
}


gboolean
menu_export_html_ok_clicked (GtkWidget *widget, gpointer data)
{
        GtkWidget *fs = GTK_WIDGET(data);
	GtkPresentation *self = static_cast<GtkPresentation*>(gtk_object_get_data(GTK_OBJECT(fs), "self"));

	char *filename = (char *)gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	
	self->presentation->ExportHTML(filename);
	
	gtk_widget_destroy(fs);	
}

gboolean
menu_export_html (gpointer data, guint callback_action, GtkWidget *widget)
{	
	GtkPresentation *self = static_cast<GtkPresentation*>(data);
	GtkWidget *fs = gtk_file_selection_new ("Export HTML");
	gtk_object_set_data(GTK_OBJECT(fs), "self", self);
	
	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
			  "clicked",
			  G_CALLBACK (menu_export_html_ok_clicked), fs);

	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
				  "clicked",
				  G_CALLBACK (gtk_widget_destroy), fs);

	gtk_widget_show (fs);
}


gboolean
menu_quit (gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	gtk_main_quit();
}


GtkPresentation::GtkPresentation (int *argc, char ***argv)
{
	idle_id = 0;

        /* Create a main window */
	gtk_init (argc, argv);
	
	mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW(mainwin), PACKAGE);
	gtk_widget_realize (mainwin);
	
	/* Presentation */
	presentation = new Presentation(0, VIDEO_NOINIT);

	/* SDL widget */	
	GtkWidget *vertical = gtk_vbox_new (FALSE, 0);	
	GtkWidget *label = gtk_label_new ("Slideshow GTK+ experimental version :-)");
	GtkWidget *quit  = gtk_button_new_with_label ("Quit");	

	/* Menu */
	static GtkItemFactoryEntry menu_items[] = {
		{ "/_File",               NULL,         NULL, 0, "<Branch>" },
		{ "/File/_Load..",        "<control>L", (GtkItemFactoryCallback)menu_load_file, 0, NULL},
		{ "/File/_Export HTML..", "<control>E", (GtkItemFactoryCallback)menu_export_html, 0, NULL},
		{ "/File/sep1",           NULL,         NULL, 0, "<Separator>" },
		{ "/File/Quit",           "<control>Q", (GtkItemFactoryCallback)menu_quit, 0, NULL},
	};

	GtkAccelGroup *accel_group = gtk_accel_group_new ();	
	GtkItemFactory *item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
	gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
	gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, this);
	gtk_window_add_accel_group (GTK_WINDOW (mainwin), accel_group);
	menu = gtk_item_factory_get_widget (item_factory, "<main>");
	
	sdl_widget = gtk_sdl_new (DEFAULT_WIDTH, DEFAULT_HEIGHT, 0, 0);
	
	g_signal_connect (G_OBJECT(mainwin), "delete_event", GTK_SIGNAL_FUNC(delete_event_cb), this);
	g_signal_connect (G_OBJECT(mainwin), "destroy", GTK_SIGNAL_FUNC(delete_event_cb), this);
	g_signal_connect (G_OBJECT(quit), "clicked", GTK_SIGNAL_FUNC(button_quit_clicked), this);
//	g_signal_connect (G_OBJECT(sdl_widget), "event", GTK_SIGNAL_FUNC(gtk_sdl_redraw), this);


	gtk_box_pack_start (GTK_BOX(vertical), menu, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vertical), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vertical), sdl_widget, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vertical), quit, FALSE, FALSE, 0);
	
	gtk_container_add (GTK_CONTAINER(mainwin), vertical);

	gtk_widget_show_all (mainwin);

	while (gtk_events_pending())
		gtk_main_iteration();

	usleep (200); // Ummmm...
	presentation->__SetScreenSurface (GTK_SDL(sdl_widget)->surface);

	idle_id = gtk_idle_add(idle_loop, this);	

	gtk_main();
}


GtkPresentation::~GtkPresentation ()
{
	if (presentation) {
		delete presentation;
	}
}


void
GtkPresentation::Load (char *filename)
{
	g_assert (filename);

	presentation->Load(filename);
}

