/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "HandDrawing.h"

#include <stdlib.h>
#include <iostream>

using namespace std;



#define HANDDRAWING_MEM_CHUNK_SIZE 256


HandDrawing::HandDrawing (unsigned color_) :
	slideshow::Drawable()
{
	   color = color_;

	   points_last   = 0;
	   points_number = 0;
	   points        = NULL; // (Uint16 *)malloc(points_number * sizeof(Uint16));
}

HandDrawing::~HandDrawing ()
{
	   if (points) {
		   free (points);
	   }
}



void
HandDrawing::AddPoint (Uint16 x, Uint16 y)
{
	   if (points_last+2 >= points_number) {
		   points_number += HANDDRAWING_MEM_CHUNK_SIZE;
		   points = (Uint16*)realloc (points, points_number * sizeof(Uint16));
	   }

	   points[points_last]   = x;
	   points[points_last+1] = y;

	   points_last += 2;
}


void
HandDrawing::Draw (SDL_Surface *screen)
{
	int x, y;
	int prevx, prevy;
	
	if (points_last < 4) return;
	prevx = points[0];
	prevy = points[1];	
	
	for (int i=2; i<points_last/2; i++) {
		x = points[(i*2)];
		y = points[(i*2)+1];

		if (x!=0xFFFF && y!=0xFFFF && prevx!=0xFFFF && prevy!=0xFFFF) {
			bresenham_line (screen, x, y, prevx, prevy);
		}
			
		prevx = x;
		prevy = y;
	}
}


void inline
HandDrawing::bresenham_line(SDL_Surface *screen, Uint32 x1, Uint32 y1, Uint32 x2, Uint32 y2)
{
	int lg_delta, sh_delta, cycle, lg_step, sh_step;

	lg_delta = x2 - x1;
	sh_delta = y2 - y1;
	lg_step = SGN(lg_delta);
	lg_delta = ABS(lg_delta);
	sh_step = SGN(sh_delta);
	sh_delta = ABS(sh_delta);
	if (sh_delta < lg_delta) {
		cycle = lg_delta >> 1;
		while (x1 != x2) {
			DrawPixel(screen, x1, y1);
			cycle += sh_delta;
			if (cycle > lg_delta) {
				cycle -= lg_delta;
				y1 += sh_step;
			}
			x1 += lg_step;
		}
		DrawPixel(screen, x1, y1);
	}
	cycle = sh_delta >> 1;
	while (y1 != y2) {
		DrawPixel(screen, x1, y1);
		cycle += lg_delta;
		if (cycle > sh_delta) {
			cycle -= sh_delta;
			x1 += lg_step;
		}
		y1 += sh_step;
	}
	DrawPixel(screen, x1, y1);
}


void inline
HandDrawing::DrawPixel (SDL_Surface *screen, Uint16 x, Uint16 y)
{
	Uint8 r = (color & 0xFF0000) >> 16;
	Uint8 g = (color & 0x00FF00) >> 8;
	Uint8 b = (color & 0x0000FF);

	Uint32 offset = ((screen->w*y)+x) * screen->format->BytesPerPixel;		
	((Uint8*)(screen->pixels))[offset]   = r;
	((Uint8*)(screen->pixels))[offset+1] = g;
	((Uint8*)(screen->pixels))[offset+2] = b;

	if (y>0) {
		y --;
		Uint32 offset = ((screen->w*y)+x) * screen->format->BytesPerPixel;		
		((Uint8*)(screen->pixels))[offset]   = r;
		((Uint8*)(screen->pixels))[offset+1] = g;
		((Uint8*)(screen->pixels))[offset+2] = b;
		y++;
	}

	if (y<screen->h) {
		y++;
		Uint32 offset = ((screen->w*y)+x) * screen->format->BytesPerPixel;		
		((Uint8*)(screen->pixels))[offset]   = r;
		((Uint8*)(screen->pixels))[offset+1] = g;
		((Uint8*)(screen->pixels))[offset+2] = b;
		y--;
	}

	if (x>0) {
		x --;
		Uint32 offset = ((screen->w*y)+x) * screen->format->BytesPerPixel;		
		((Uint8*)(screen->pixels))[offset]   = r;
		((Uint8*)(screen->pixels))[offset+1] = g;
		((Uint8*)(screen->pixels))[offset+2] = b;
		x++;
	}

	if (x<screen->w) {
		x++;
		Uint32 offset = ((screen->w*y)+x) * screen->format->BytesPerPixel;		
		((Uint8*)(screen->pixels))[offset]   = r;
		((Uint8*)(screen->pixels))[offset+1] = g;
		((Uint8*)(screen->pixels))[offset+2] = b;
		x--;
	}


}


string
HandDrawing::ToPython (void) const
{
	return "";
}


string
HandDrawing::ToXML (void) const
{
	return "";
}


string
HandDrawing::ToCPP (void) const
{
	return "";
}
