/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <stdio.h>
#include <unistd.h>

#include "HtmlExporter.h"
#include "Macros.h"
#include "Utilities.h"
#include "Bmp2Png.h"

#include <string>
#include <fstream>
#include <iostream>

#include <SDL_image.h>
#include <sge.h>


using namespace std;



HtmlExporter::HtmlExporter () :
	Exporter ()
{
}

HtmlExporter::~HtmlExporter()
{
}

void
HtmlExporter::Write (Presentation *pre, const char *prefix)
{
	char  fn_bmp[512];	
	char  fn_png[512];
	char *filename_bmp = &fn_bmp[0];
	char *filename_png = &fn_png[0];
	
	SS_ASSERT (pre);

	// Write bmp files
	snprintf (filename_bmp, 512, "%stmp_image", prefix);
	int n = pre->WriteBMPs(filename_bmp);

        // Convert images
	for (int i=1; i<n; i++) {
		// conversion
		snprintf (filename_bmp, 512, "%stmp_image%03i.bmp", prefix, i);
		snprintf (filename_png, 512, "%simage%03i.png", prefix, i);
		ConvertBMP2PNG (filename_bmp, filename_png);

		// Make the thumbnail
		snprintf (filename_png, 512, "%simage%03i-thumb", prefix, i);
		SaveThumbnail (filename_bmp, filename_png);

		// remove bmp
		unlink (filename_bmp);


                // Write de HTML
		snprintf (filename_bmp, 512, "%simage%03i", prefix, i);
		WriteHTML (i, n, filename_bmp);
	}

	// Index file
	WriteIndex (n, prefix);
}


void
HtmlExporter::WriteHTML (int i, int total, const char *path_wo_extension)
{
	char    buf_[512];
	char   *buf = &buf_[0];
	string  html_filename;

	html_filename = path_wo_extension;
	html_filename +=".html";
	
	ofstream html(html_filename.c_str());

	html << HTML_EXPORTER_HEADER << endl;
	html << "<p>" << endl;
	
	if (i <= 1) {
		html << "&#91;<b>&lsaquo;&lsaquo;</b> First&#93; &#91;<b>&lsaquo;</b> Prev&#93; ";
	} else {
		html << "&#91;<a href=\"image001.html\"><b>&lsaquo;&lsaquo;</b> First</a>&#93; ";	
		snprintf (buf, 512, "%03i", i-1);
		html << "&#91;<a href=\"image" << buf << ".html\"><b>&lsaquo;</b> Prev</a>&#93; ";
	}

	html << "&#91;<a href=\"index.html\">Index</a>&#93; ";

	if (i < total-1) {
		snprintf (buf, 512, "%03i", i+1);
		html << "&#91;<a href=\"image" << buf << ".html\">Next <b>&rsaquo;</b></a>&#93; ";
		snprintf (buf, 512, "%03i", total-1);
		html << "&#91;<a href=\"image" << buf << ".html\">Last <b>&rsaquo;&rsaquo;</b></a>&#93; ";	
	} else {
		html << "&#91;<b>&rsaquo;</b> Next&#93; &#91;<b>&rsaquo;&rsaquo;</b> Next&#93;";
	}

	
	html << "	</p>" << endl
	     <<	"	<hr />" << endl
	     << "	<p>" << endl
	     << "	<img alt=\"\" src=\"" << path_wo_extension << ".png" << "\" />" << endl
	     << "	</p><hr /><p>" << endl
	     << "	Generated by <a href=\"http://www.alobbs.com/slideshow\">slideshow</a>" << endl
	     << "</p>" << endl
	     << HTML_EXPORTER_FOOT << endl;
	html.close();
}


void
HtmlExporter::SaveThumbnail (const char *f_bmp, const char *png_path_wo_suf)
{
	string filename;
	
	SDL_Surface *image = IMG_Load (f_bmp);
	if (!image) {
		cerr << "ERROR: Can't " << f_bmp << endl;
		return;
	}

	SDL_Surface *thumb = sge_transform_surface(image,
						   0,   /* color */
						   0,   /* angle */
						   0.4, /* x scale */
						   0.4, /* y scale */
						   0);  /* flags */

	filename = png_path_wo_suf;
	filename += ".bmp";

	if (SDL_SaveBMP (thumb, filename.c_str()) < 0) {
		cerr << "ERROR: Cannot write " << filename.c_str() << endl;
		return;
	}

	string png;
	png = png_path_wo_suf;
	png += ".png";

	ConvertBMP2PNG (filename.c_str(), png.c_str());

	unlink (filename.c_str());
	
	SDL_FreeSurface (image);
	SDL_FreeSurface (thumb);
}


void
HtmlExporter::WriteIndex (int total, const char* prefix)
{
	string index;
	index = prefix;
	index += "index.html";
	
	ofstream html(index.c_str());

	html << HTML_EXPORTER_HEADER << endl;
	html << "<p>" << endl;

	
	char buf_[200];
	char *buf = &buf_[0];
	
	for (int i=1; i<total; i++) {
		snprintf (buf, 200, "image%03i.html", i);
		html << "<a href=\"" << buf << "\">";
		snprintf (buf, 200, "Slide %i", i);
		html << "<img alt=\"" << buf << "\" ";
		snprintf (buf, 200, "image%03i-thumb.png", i);
		html << "src=\"" << buf << "\" /></a>&nbsp;&nbsp;" << endl;
	}

	html << "</p>" << endl;
	html << HTML_EXPORTER_FOOT << endl;
	
	html.close();
}

