#ifndef __SlideShow_Macros__
#define __SlideShow_Macros__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Constants
//
#define SS_UNDEFINED       -1
#define SS_UNDEFINED_INT   0xDEADBEAF
#define SS_UNDEFINED_COLOR SS_UNDEFINED_INT
#define SS_UNDEFINED_POS   SS_UNDEFINED_INT

// Type for introspection
//
#include "Types.h"
#define SET_CLASS(x)      virtual SlideShow_ClassType __SS_IntrospectionGetClassType(void) { return ClassType_ ## x; }
#define GET_CLASS(x)      x->__SS_IntrospectionGetClassType()
#define GET_CLASS_NAME(x) SlideShow_ClassType2string(GET_CLASS(x))

// Debug
//
#ifdef DEBUG
#include <assert.h>
#define SS_ASSERT(x) assert(x)
#else
#define SS_ASSERT(x)
#endif

// Macro-functions
//
#ifndef MAX
#define MAX(x,y) (x>y?x:y)
#endif // MAX

#ifndef MIX
#define MIN(x,y) (x>y?y:x)
#endif // MIN

#ifndef SGN
#define SGN(x) ((x)>0 ? 1 : ((x)==0 ? 0:(-1)))
#endif // Sgn

#ifndef ABS
#define ABS(x) ((x)>0 ? (x) : (-x))	
#endif // ABS

#ifndef PI
#define PI 3.14159265358979323846
#endif // PI


#define COLOR_TO_RGB(pixel,r,g,b) \
	if(SDL_BYTEORDER == SDL_BIG_ENDIAN) { \
	    r = (pixel >> 16) & 0xff; \
	    g = (pixel >> 8) & 0xff; \
	    b = pixel & 0xff; \
	} else { \
	    r = pixel & 0xff; \
	    g = (pixel >> 8) & 0xff; \
	    b = (pixel >> 16) & 0xff; \
	} 

#define GET_PIXEL(surface,x,y,color) { \
	   Uint8 bpp = surface->format->BytesPerPixel; \
	   Uint8  *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp; \
	   switch(bpp) { \
	   case 1: color = *p; break; \
	   case 2: color = *(Uint16 *)p; break; \
	   case 3: \
			 if(SDL_BYTEORDER == SDL_BIG_ENDIAN) { \
				    color = p[0] << 16 | p[1] << 8 | p[2]; \
			 } else { \
				    color = p[0] | p[1] << 8 | p[2] << 16; \
			 } break; \
	   case 4: color = *(Uint32 *)p; break; \
	   } \
}


#define PUT_PIXEL(screen,x,y,r,g,b) { \
	Uint32 color = SDL_MapRGB(screen->format,r,g,b); \
	switch (screen->format->BytesPerPixel) { \
	  case 1: ((Uint8  *)screen->pixels)[y*screen->pitch + x] = color; break; \
	  case 2: ((Uint16 *)screen->pixels)[y*screen->pitch/2 + x] = color; break; \
	  case 3: { Uint8 * bufp = &((Uint8 *)(screen->pixels))[y*screen->pitch + x * 3]; \
		   if(SDL_BYTEORDER == SDL_LIL_ENDIAN) { \
				 bufp[0] = color; \
				 bufp[1] = color >> 8; \
				 bufp[2] = color >> 16; \
		   } else { \
				 bufp[2] = color; \
				 bufp[1] = color >> 8; \
				 bufp[0] = color >> 16; \
		   } } \
	  	   break; \
	  case 4: ((Uint32 *)screen->pixels)[y*screen->pitch/4 + x] = color; break; \
	} \
}



#endif // __SlideShow_Macros__ 
