/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "Mpeg.h"
#include <iostream>
#include <assert.h>

using namespace std;



Mpeg::Mpeg (const char *filename) :
	   slideshow::Drawable()
{
	   mpeg = NULL;
	   LastMagicNum = 0xFFFFFFFF;

	   if (filename != NULL) {
			 Load(filename);
	   }
}


Mpeg::~Mpeg ()
{
	   if (mpeg != NULL) {
			 SMPEG_delete(mpeg);
	   }
}


void
DisplayCallback (SDL_Surface* dst, int x, int y,
		 unsigned int w, unsigned int h)
{}


void
Mpeg::Load (const char *filename)
{
	   if (mpeg != NULL) {
			 SMPEG_delete(mpeg);
	   }

	   mpeg = SMPEG_new (filename, &info, false); // audio
	   if ( SMPEG_error(mpeg) ) {
		   cerr << filename << ": " << SMPEG_error(mpeg) << endl;
		   SetErrorBox();
		   return;
	   }
	   
	   SMPEG_enableaudio (mpeg, 0);
	   
	   if (!info.has_video) {
		   cerr << filename << ": hasn't video" << endl;
		   SetErrorBox();
		   SMPEG_delete(mpeg);
		   return;
	   }

	   // Testing
	   SMPEG_loop(mpeg, 1);
	   //

	   /*
	   filter = SMPEGfilter_bilinear();
	   filter = SMPEG_filter( mpeg, filter );
	   filter->destroy(filter);
	   */
	   
	   surface = SDL_CreateRGBSurface (SDL_SWSURFACE, info.width, info.height, 24,
					   0x0000FF, 0x00FF00, 0xFF0000, 0);

	   SMPEG_setdisplay(mpeg, surface, NULL, DisplayCallback);
	   SMPEG_enablevideo (mpeg, 1);
	   SMPEG_play (mpeg);

	   need_redraw = true;
}

void
Mpeg::SetErrorBox (void)
{
	assert (surface == NULL);
	
	surface = SDL_CreateRGBSurface (SDL_SWSURFACE, 100, 100, 24,
					0x0000FF, 0x00FF00, 0xFF0000, 0);
	for (int j=0; j<100; j++) {
		int k = j*3;
		((Uint8*)(surface->pixels))[(100*k)+k] = 0xFF;
		((Uint8*)(surface->pixels))[(100*k)+k+1] = 0xFF;
		((Uint8*)(surface->pixels))[(100*k)+k+2] = 0xFF;
		
		((Uint8*)(surface->pixels))[(100*k)-k] = 0xFF;
		((Uint8*)(surface->pixels))[(100*k)-k+1] = 0xFF;
		((Uint8*)(surface->pixels))[(100*k)-k+2] = 0xFF;
	}
}



bool
Mpeg::NeedRedraw (void)
{
	   if (Drawable::NeedRedraw()) return true;

	   Uint32 mn = CalculateMagicNum();
	   if (LastMagicNum != mn) {
			 LastMagicNum = mn;
			 return true;
	   }

	   return false;
}


Uint32
Mpeg::CalculateMagicNum(void)
{
	   Uint32 num = 0;
	   
	   for (int j=0; j<surface->h; j++) {
			 num += ((Uint8 *)(surface->pixels))[(j * surface->h) + j];
	   }

	   return num;
}
	   

string
Mpeg::ToPython (void) const
{
	return "";
}


string
Mpeg::ToXML (void) const
{
	return "";
}


string
Mpeg::ToCPP (void) const
{
	return "";
}



//
// Events
//

void
Mpeg::On_JoiningPage (void)
{
	if (mpeg) {
		SMPEG_play(mpeg);
	}
}

void
Mpeg::On_LeavingPage (void)
{
	if (mpeg) {
		SMPEG_stop(mpeg);
	}
}


