/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __Page__
#define __Page__

#include <SDL.h>

#include <list>
#include <string>
#include <fstream>
#include <iostream>

#include "Macros.h"
#include "Drawable.h"
#include "Image.h"
#include "Text.h"
#include "TextDouble.h"
#include "Scroll.h"
#include "Writer.h"
#include "FontVault.h"
#include "Gradient.h"
#include "Stars.h"
#include "Plasma.h"
#include "Eraser.h"
#include "Fires.h"
#include "EffectDarkness.h"
#include "Spinner.h"
#include "Mpeg.h"
#include "HandDrawing.h"


using namespace std;

class Presentation;

typedef list<slideshow::Drawable*> DrawableList;


class Page {
public:
	SET_CLASS (Page);

	Uint32 Number;
	   
	Page (Page *base = NULL);
	~Page();

	void Add  (slideshow::Drawable *item);
	void Draw (SDL_Surface *screen);
	void Save (SDL_Surface *screen, const char *filename);

	string ExportPython (void) const;
	string ExportXML    (void) const;
	string ExportCPP    (void) const;
	   
	void SetBase        (Page *base);
	void SetPresentaion (Presentation *presentation);

	bool NeedRedraw (void);

	bool FirstStage (void);
	bool LastStage  (void);	
	bool StageFinished (void);
	void SetPrevStage (void);
	void SetNextStage (void);
	void SetFisrtStage (void);

	void NotifyLeaving (void);
	void NotifyJoining (void);

	void DrawingAddPoint (Uint16 x, Uint16 y);

	   
	// Helper methods
	void        AddNote (const char *note);

	Image      *AddNewImage         (char *filename, int x=-1, int y=-1);
	Image      *AddNewImageScrolled (char *filename, int dir, int another, int stop_at);
	   
	Text       *AddNewText         (char *fontname, char *text=NULL, int x=-1, int y=-1, int color=0);	
	Text       *AddNewTextScrolled (char *fontname, char *text, int dir, int another, int stop_at, int color=0);

	TextDouble *AddNewTextDouble         (char *fontname, char *text=NULL, int x=-1, int y=-1, int color=0, int color2=0xFFFFFF);
	TextDouble *AddNewTextDoubleScrolled (char *fontname, char *text, int dir, int another, int stop_at, int c1=0, int c2=0xFFFFFF);

	Mpeg       *AddNewMpeg (char *filename, int x=-1, int y=-1);
	Mpeg       *AddNewMpegScrolled (char *filename, int dir, int another, int stop_at);	   
	   
	void        AddStop        (float secs = 0);
	Stars      *AddNewStars    (void);
	Plasma     *AddNewPlasma   (void);
	Fires      *AddNewFires    (void);
	Eraser     *AddNewEraser   (unsigned int color=0);
	Gradient   *AddNewGradient (unsigned int color1, unsigned int color2,SlideShowGradient type=GRADIENT_VERTICAL);

	EffectDarkness *AddNewEffectDarkness (slideshow::Drawable *drawable);
           
	Spinner    *AddNewSpinner(slideshow::Drawable *drawable, int degrees_second=90);
	   
	Writer     *AddNewWriter (char *font=NULL, int x=SS_UNDEFINED_POS, int y=SS_UNDEFINED_POS,
				  int color=SS_UNDEFINED_COLOR, int color2=SS_UNDEFINED_COLOR);
	Page       *operator+=   (char *text);
	   
protected:
	Page            *base;
	Presentation    *owned;
	   
	DrawableList     drawables;
	Writer          *last_writer;
	HandDrawing     *drawing;
	string           notes;
	   
	int              stage_adding;
	int              stage_current;
	time_t           stage_current_begin;
	int              stage_max;	   
	vector<float>    times;
};


#endif // __Page__
