/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include <time.h>
#include <math.h>
#include <stdlib.h>

#include "Macros.h"
#include "Plasma.h"



Plasma::Plasma (int screen_width, int screen_height, int elapse_) :
	slideshow::Drawable()
{
	Uint32 rmask, gmask, bmask, amask;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
	amask = 0x000000ff;
#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
#endif	
	   	   
	surface = SDL_CreateRGBSurface (SDL_SWSURFACE, screen_width, screen_height, 8,
					rmask, gmask, bmask, amask);

	elapse = elapse_;
	state = 0;
	last_time = clock();
	   
	dest.x = 0;
	dest.y = 0;
	dest.w = surface->w;
	dest.h = surface->h;

	VIDEOX = screen_width;
	VIDEOY = screen_height;
	TABLEX = VIDEOX * 2;
	TABLEY = VIDEOY * 2;
	   
	   
	srand(time(NULL));
	{
		int c;
		for (c=0; c<3; c++)
			r[c]=((double)(rand()%1000+1))/300000;
		for (c=0; c<6; c++)
			R[c]=((double)(rand()%1000+1))/5000;
	}

	t = (unsigned char *)malloc(TABLEY*TABLEX);

	{
		int y;
		for (y=0 ; y<TABLEY ; y++) {
			int x;
			for (x=0 ; x<TABLEX ; x++) {
				double tmp=(((double)((x-(TABLEX/2))*(x-(TABLEX/2))+(y-(TABLEX/2))*(y-(TABLEX/2))))
					    *(PI/(TABLEX*TABLEX+TABLEY*TABLEY)));
				t[y*TABLEX+x]=(sin(sqrt(tmp)*12)+1)*TONES/6;
			}
		}
	}
}

Plasma::~Plasma()
{
	if (t) {
		free(t);
	}
	   
	if (surface) {
		SDL_FreeSurface(surface);
	}
}


void
Plasma::Draw (SDL_Surface *screen)
{
	clock_t now = clock();
	   
	if (now - last_time > elapse) {
		last_time = now;
		state++;
	   
		for (int c=0 ; c<TONES; c++) {
			colors[c].r=(sin(((double)c)/TONES*6*PI+r[0]*PI*state*PI)+1)*127;
			colors[c].g=(sin(((double)c)/TONES*6*PI+r[1]*state*PI)+1)*127;
			colors[c].b=(sin(((double)c)/TONES*6*PI+r[2]*state*PI)+1)*127;
		}
		SDL_SetColors(surface, colors, 0, TONES);


		SDL_LockSurface (surface);
		do_plasma(surface,
			  (sin(((double)state)*R[0])+1)/2,
			  (sin(((double)state)*R[1])+1)/2,
			  (sin(((double)state)*R[2])+1)/2,
			  (sin(((double)state)*R[3])+1)/2,
			  (sin(((double)state)*R[4])+1)/2,
			  (sin(((double)state)*R[5])+1)/2, t);
		SDL_UnlockSurface(surface);
	}
	   

	SDL_BlitSurface (surface, NULL, screen, &dest);

	// FIXME: testing
	need_redraw = true;
}


void
Plasma::do_plasma (SDL_Surface *surface,
		   double x1,
		   double y1,
		   double x2,
		   double y2,
		   double x3,
		   double y3,
		   unsigned char*t)
{
	unsigned int
		X1=x1*(TABLEX/2),
		Y1=y1*(TABLEY/2),
		X2=x2*(TABLEX/2),
		Y2=y2*(TABLEY/2),
		X3=x3*(TABLEX/2),
		Y3=y3*(TABLEY/2);

	unsigned int y;
	unsigned char *t1=t+X1+Y1*TABLEX, *t2=t+X2+Y2*TABLEX, *t3=t+X3+Y3*TABLEX;
	for (y=0; y < VIDEOY; y++) {
		unsigned x;
		unsigned char*tmp= ((Uint8*)surface->pixels) + y * surface->pitch;
		unsigned int t=y*TABLEX, tmax=t+VIDEOX;
		for (x=0; t<tmax; t++, tmp++) {
			tmp[0]=t1[t]+t2[t]+t3[t];
		}
	}
}


string
Plasma::ToPython (void) const
{
	return "";
}


string
Plasma::ToXML (void) const
{
	return "";
}


string
Plasma::ToCPP (void) const
{
	return "";
}
