/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __Plasma__
#define __Plasma__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <SDL.h>
#include "Drawable.h"


#define TONES 256


class Plasma : public slideshow::Drawable {
public:
	   SET_CLASS(Plasma);
	   
	   Plasma (int screen_width=DEFAULT_WIDTH, int screen_height=DEFAULT_HEIGHT, int elapse_=400000);
	   ~Plasma();

	   virtual void Draw (SDL_Surface *screen);

	   virtual string ToPython (void) const;
	   virtual string ToXML    (void) const;
	   virtual string ToCPP    (void) const;

	   
private:
	   SDL_Rect dest;

	   int state;	
	   int elapse;
	   clock_t last_time;

	   double r[3];
	   double R[6];
	   int aSin[512];
	   unsigned char *t;
	   SDL_Color colors[TONES*2];

	   
	   int VIDEOX, VIDEOY;
	   int TABLEX, TABLEY;

	   void do_plasma (SDL_Surface *surface,
				    double x1,
				    double y1,
				    double x2,
				    double y2,
				    double x3,
				    double y3,
				    unsigned char*t);
};

#endif // __Plasma__

