/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __Presentation__
#define __Presentation__

#include <list>
#include <SDL.h>
#include "Page.h"
#include "FontVault.h"


#define VIDEO_NOINIT -1
#define VIDEO_DEFAULT 0
#define VIDEO_ASCII   1


class Presentation {
public:
	Presentation (int extra_sdl_flags = 0,
		      int video_mode      = VIDEO_DEFAULT,
		      int xsz             = DEFAULT_HEIGHT, 
		      int ysz             = DEFAULT_WIDTH);
	~Presentation();
	
	FontVault Fonts;
	Uint32    Last_writer_color1;
	Uint32    Last_writer_color2;	   
	Uint32    Last_writer_posx;
	Uint32    Last_writer_posy;
	char     *Last_writer_fontname;
	
	// Very high level methods
	void Load        (const char *filename);
	void Run         (void);
	
	void ExportHTML   (const char *directory);
	void ExportPython (const char *filename);
	void ExportXML    (const char *filename);
	void ExportCPP    (const char *filename);

	bool RequireVersion (unsigned short int version, 
			     unsigned short int subversion, 
			     unsigned short int microversion);
	
	// The rest
	void  Add        (Page *page);
	Page *AddNewPage (Page *base = NULL);
	bool  IsEmpty    (void);
	
	void SetNextPage (void);
	void SetPrevPage (void);
	
	void SetNextStage (void);
	void SetPrevStage (void);
	
	int  WriteBMPs (const char *prefix = NULL);
	void WriteCurrentBMP (const char *name = NULL);
	
	void MouseSetActive (void);
	void MouseSetInactive (void);

	void ExecuteCommand (const char *command);
	
	bool DoMainloopIteration (void);
	
	// FIXME:
	void __SetScreenSurface (SDL_Surface *surface);
	
	int GetWidth() { return width; };
	int GetHeight() { return height; };
	   
private:
	SDL_Surface      *screen;
	std::list<Page*>  pages;
	Page             *page_actual;
	bool              force_redraw_page;
	bool              mouse_is_active;
	Uint32            last_page_number;

	int               width;
	int               height;
};


#endif // __Presentation__
