/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "Macros.h"
#include "ProgressiveShow.h"


ProgressiveShow::ProgressiveShow (Drawable *parent, int pixels_per_second) :
	   Effect (parent)
{
	SS_ASSERT (parent);
	
	x_pos = 0;
	pps = pixels_per_second;
}

void
ProgressiveShow::ProcessEffect (void)
{
	   if (x_pos < parent->Width()) {
			 x_pos += pps;
			 if (x_pos > parent->Width()) {
				    x_pos = parent->Width();
			 }
	   }

	   rect.x = 0;
	   rect.y = 0;
	   rect.w = x_pos;
	   rect.h = parent->Height();
	   
	   dest.x = x;
	   dest.y = y;
	   dest.w = x_pos;
	   dest.h = parent->Height();
}


void
ProgressiveShow::Draw (SDL_Surface *screen)
{
	   if (x_pos < parent->Width()) {
			 ProcessEffect();
	   }
	   
	   SDL_BlitSurface (parent_surface, &rect, screen, &dest);
}


string
ProgressiveShow::ToPython (void) const
{
	return "";
}


string
ProgressiveShow::ToXML (void) const
{
	return "";
}


string
ProgressiveShow::ToCPP (void) const
{
	return "";
}


void
ProgressiveShow::SayCheese (void)
{
	x_pos = parent->Width();
}
