/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Scroll.h"



Scroll::Scroll (int w, int h, int *x_, int *y_, int direction_, int stop_at_, int another_pos, int pps_)
{
	x         = x_;
	y         = y_;
	direction = direction_;
	stop_at   = stop_at_;
	pps       = pps_;
	finished  = false;
	begin.tv_sec = begin.tv_usec = 0;

	   
	switch (direction) {
	case SOUTH_NORTH:
		(*y) = start_at = DEFAULT_HEIGHT + 1;
		if (another_pos != -1) (*x) = another_pos;
		break;

	case NORTH_SOUTH:
		(*y) = start_at = - (h + 1);
		if (another_pos != -1) (*x) = another_pos;
		break;

	case EAST_WEST:
		(*x) = start_at = DEFAULT_WIDTH + 1;
		if (another_pos != -1) (*y) = another_pos;
		break;

	case WEST_EAST:
		(*x) = start_at = - (w + 1);
		if (another_pos != -1) (*y) = another_pos;
		break;
	}
}

//
#include <iostream>
using namespace std;
//


void
Scroll::Move (void)
{
	if (finished) {
		return;
	}

	if ((begin.tv_sec == 0) && (begin.tv_usec == 0)) {
		gettimeofday(&begin, NULL);
	}

	struct timeval now;
	gettimeofday(&now, NULL);

	double elapse = (((double)now.tv_sec + (double)now.tv_usec/(1000*1000)) - 
			 ((double)begin.tv_sec + (double)begin.tv_usec/(1000*1000)));
		
	int offset = elapse * pps;
 	   
	switch (direction) {
	case SOUTH_NORTH:
		if ((*y) > stop_at) {
			(*y) = (start_at-offset > stop_at) ? start_at-offset : stop_at;
		} else {
			finished = true;
		}
		break;

	case NORTH_SOUTH:
		if ((*y) < stop_at) {
			(*y) = (start_at+offset < stop_at) ? start_at+offset : stop_at;
		} else {
			finished = true;
		}
		break;

	case EAST_WEST:
		if ((*x) > stop_at) {
			(*x) = (start_at-offset > stop_at) ? start_at-offset : stop_at;
		} else {
			finished = true;
		}
		break;

	case WEST_EAST:
		if ((*x) < stop_at) {
			(*x) = (start_at+offset < stop_at) ? start_at+offset : stop_at;
		} else {
			finished = true;
		}
		break;
	}
}


bool
Scroll::Finished (void)
{
	return finished;
}


void
Scroll::MakeFinish (void)
{
	switch (direction) {
	case SOUTH_NORTH:
	case NORTH_SOUTH:
		*y = stop_at;
		break;
	case EAST_WEST:
	case WEST_EAST:
		*x = stop_at;
		break;
	}
}
