/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*
 * This portion based on stars.c by Nathan Strong
 * <gblues@gstis.net>
 */


#include "Stars.h"
#include <stdlib.h>	
#include <string.h>


Stars::Stars (int screen_width, int screen_height) :
	slideshow::Drawable()
{
	SCREEN_WIDTH = screen_width;
	SCREEN_HEIGHT = screen_height;
	
	for (int i = 0; i < NUMBER_OF_STARS; i++) {
		init_star(stars + i, i + 1);
	}
}


Stars::~Stars()
{
}


void
Stars::Draw (SDL_Surface *screen)
{
	Uint16 centerx, centery;
	int tempx, tempy;
	
	centerx = SCREEN_WIDTH >> 1;
	centery = SCREEN_HEIGHT >> 1;
	
//	memset(screen->pixels, 0, SCREEN_WIDTH * SCREEN_HEIGHT * sizeof(Uint32));
	
	
	for (int i = 0; i < NUMBER_OF_STARS; i++)
	{
		stars[i].zpos -= stars[i].speed;
		
		if (stars[i].zpos <= 0)
		{
			init_star(stars + i, i + 1);
		}
		
		// compute 3D position
		tempx = (int) (stars[i].xpos / stars[i].zpos) + centerx;
		tempy = (int) (stars[i].ypos / stars[i].zpos) + centery;
		
		if (tempx < 0 || tempx > SCREEN_WIDTH - 1 || tempy < 0 || tempy > SCREEN_HEIGHT - 1)
		{
			init_star(stars + i, i + 1);
			continue;
		}
		
		((Uint32*)screen->pixels)[(tempy*SCREEN_WIDTH) + tempx] = 0xffffffff;
	}
	
}


void
Stars::init_star (STAR* star, int i)
{
	   star->xpos =  -10.0 + (20.0 * (rand()/(RAND_MAX+1.0)));
	   star->ypos =  -10.0 + (20.0 * (rand()/(RAND_MAX+1.0)));
	   
	   star->xpos *= 3072.0; // change viewpoint
	   star->ypos *= 3072.0;
	   
	   star->zpos =  i;
	   star->speed =  2 + (int)(2.0 * (rand()/(RAND_MAX+1.0)));
	   
	   star->color = i >> 2; // the closer to the viewer the brighter
}


string
Stars::ToPython (void) const
{
	return "";
}


string
Stars::ToXML (void) const
{
	return "";
}


string
Stars::ToCPP (void) const
{
	return "";
}
