/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "TextDouble.h"


TextDouble::TextDouble (TTF_Font *font, int color1, int color2) :
	   slideshow::Drawable()
{
	   text1.SetFont (font);
	   text2.SetFont (font);

	   text1.SetColor ((color1 & 0xFF0000) >> 16, (color1 & 0x00FF00) >> 8, (color1 & 0x0000FF));
	   text2.SetColor ((color2 & 0xFF0000) >> 16, (color2 & 0x00FF00) >> 8, (color2 & 0x0000FF));
}


void
TextDouble::Draw (SDL_Surface *screen)
{
	   text1.x = x;
	   text1.y = y;
	   text2.x = x+2;
	   text2.y = y-2;

	   if ((scroll) && (!scroll->Finished())) {
		   scroll->Move();
	   }
	   
	   text1.Draw(screen);
	   text2.Draw(screen);
}


void
TextDouble::SetText (char *txt)
{
	   text1 = txt;
	   text2 = txt;
}
void TextDouble::operator= (char *txt) { SetText(txt); }


void
TextDouble::AppendText (char *txt)
{
	   text1 += txt;
	   text2 += txt;	   
}
void TextDouble::operator+= (char *txt) { AppendText(txt); }


void
TextDouble::SetStyle (bool bold, bool italic, bool underline)
{
	   text1.SetStyle (bold, italic, underline);
	   text2.SetStyle (bold, italic, underline);
}


Uint16
TextDouble::Width (void)
{
	   return text1.Width();
}


Uint16
TextDouble::Height (void)
{
	   return text1.Height();
}


bool
TextDouble::NeedRedraw (void)
{
	bool tmp = (text1.NeedRedraw() || text2.NeedRedraw());  

	if (scroll) {
		tmp |= (!scroll->Finished());
	}

	return tmp;
}


string
TextDouble::ToPython (void) const
{
	return "";
}


string
TextDouble::ToXML (void) const
{
	return "";
}


string
TextDouble::ToCPP (void) const
{
	return "";
}
