/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "Utilities.h"
#include "Macros.h"

#include <iostream>


/*
 * String utils
 */

bool
endsWith (std::string str, std::string suffix)
{
	std::string::size_type i = str.rfind( suffix );
	if (i == std::string::npos)
		return false;
	if (i == ( str.size() - suffix.size()))
		return true;
	return false;
}


bool
split(std::vector<std::string> &vec, const std::string &str,const std::string &delim)
{
	   vec.clear();

	   if (delim.empty())
	   {
			 vec.push_back(str);
			 return false;
	   }

	   std::string::size_type i = 0;
	   std::string::size_type j = 0;

	   for (;;)
	   {
			 j = str.find(delim,i);
			 if (j==std::string::npos)
			 {
				    vec.push_back(str.substr(i));
				    break;
			 }

			 vec.push_back(str.substr(i,j-i));
			 i = j + delim.size();

			 if (i==str.size())
			 {
				    vec.push_back(std::string());
				    break;
			 }
	   }

	   return true;
}







/*
 * Path utils
 */

std::string
getDirName (std::string fullpath)
{
	   std::string::size_type pos = fullpath.rfind('/');
	   
	   if (pos != std::string::npos) {
			 return fullpath.substr(0, pos);
	   }
	   return fullpath;
}


std::string
programInPath (std::string program)
{
	   std::string ret = "";
	   std::vector<std::string> dirs;
	   std::string path = getenv("PATH");
	   struct stat file_stat;
	   
	   if (!split(dirs, path, ":")) {
			 return ret;
	   }

	   for (std::vector<std::string>::iterator it = dirs.begin(); it != dirs.end(); ++it) {
			 std::string fullpath;
			 bool is_ok;
			 fullpath = (*it) + "/" + program;

			 is_ok = (stat (fullpath.c_str(), &file_stat) == 0);
			 is_ok &= file_stat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH);

			 if (is_ok) {
				    ret = fullpath;
				    break;
			 }
	   }

	   return ret;
}
