/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <iostream>

#include "Text.h"
#include "Writer.h"
#include "TextDouble.h"

using namespace std;


Writer::Writer(FontVault *fontvault_, int color, int color2) :
	slideshow::Drawable()
{
	   fontvault = fontvault_;
	   default_font = NULL;
	   xmax = 0;
	   ymax = 0;
	   last_color  = color;
	   last_color2 = color2;
}


Writer::~Writer()
{
}


void
Writer::AppendLine (char *text, int color, char *font)
{
	   TTF_Font *ttf = NULL;

	   if (font) {
			 ttf = (*fontvault)[font];
	   }

	   if ((!ttf) && default_font) {
			 ttf = default_font;
	   }

	   if (!ttf) {
			 ttf = fontvault->GetFirstOne();
	   }

	   if (!ttf) {
			 cerr << "Unknown font '" << font << "'" << endl;
			 return;
	   }
	   
	   if (color != SS_UNDEFINED_COLOR) {
		   last_color = color;
	   }
	   
	   Drawable *nd;
	   if  (last_color2 != SS_UNDEFINED_COLOR) {
			 TextDouble *nt = new TextDouble(ttf, last_color, last_color2);
			 nt->SetText(text);	   
			 nd = nt;
	   } else {
			 Text *nt = new Text(ttf);
			 nt->SetColor (last_color);
			 nt->SetText(text);	   
			 nd = nt;
	   }

	   nd->x = x;
	   nd->y = y + ymax;
	   
	   drawables.push_back (nd);

	   int maybe_xmax = nd->Width();
	   if (maybe_xmax > xmax) {
			 xmax = maybe_xmax;
	   }
	   ymax += nd->Height();
}


void
Writer::Draw (SDL_Surface *screen)
{
	   for (list<Drawable*>::iterator it=drawables.begin(); it!=drawables.end(); ++it) {
			 (*it)->Draw(screen);
	   }

	   need_redraw = false;
}


Uint16
Writer::Height (void)
{
	   return ymax;
}


Uint16
Writer::Width (void)
{
	   return xmax;
}


void
Writer::SetDefaultFont (char *font)
{
	   default_font = (*fontvault)[font];
}


string
Writer::ToPython (void) const
{
	return "";
}


string
Writer::ToXML (void) const
{
	return "";
}


string
Writer::ToCPP (void) const
{
	return "";
}
