/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *	Abraham Carrero Garcia (abecin@terra.es)
 *
 * Copyright (C) 2002-2003 Abraham Carrero Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
 
 #include "XmlLoader.h"
 
 XmlLoader::XmlLoader() {
 templates.clear();
 }
 
  XmlLoader::~XmlLoader() { 
    for (Templates_HashMap::iterator it = templates.begin(); it!=templates.end(); ++it) {
       delete((*it).second);
    }  
  
  }
 
 

 /**
  * xmlParseFile(char* fileName) Reads file and stores it in memory
  * xmlNodeListGetStr(XmlDocPtr, xmlNodePtr, int) Returns node's text
  * xmlGetProp(xmlNodePtr ,const xmlChar *) Returns text associated to the property whose name is represented by const xmlChar
  */
 void XmlLoader::LoadInto (const char *fileName, Presentation *load_into)
 {

	xmlDocPtr doc;
	xmlNodePtr cur;
 	
	//Presentation *pres = NULL;

        cout<<"Reading ["<<fileName<<"]"<<endl;
        
	doc = xmlParseFile(fileName); //load the document
	cout<<"File ["<<fileName<<"] parsed"<<endl;
        
	if(doc == NULL){
	cerr<<"Failed to load the document ["<<fileName<<"]"<<endl;
	xmlFreeDoc(doc);//free the document
	return;
	}//if(doc == NULL)
	
	cur = xmlDocGetRootElement(doc);//We first get the root node
	
	if (cur == NULL) {
		cerr<<"Empty document"<<endl;
		xmlFreeDoc(doc);
		return;
	}	
	

	cout<<"First Level: ["<<cur->name<<"]"<<endl;
	

        if (!xmlStrcmp(cur->name, (const xmlChar *) "slideshow"))
	{
//          pres = parseSlideShow(doc,cur,maybe_presentation);	
            parseSlideShow(doc,cur,load_into);
	    }
        else
            {
                cout<<"ROOT element must be slideshow, found: "<<cur->name;
                return;
            }
        
/*	while(cur != NULL){
		if(cur->xmlChildrenNode != NULL)
			cout<<"	"<<cur->xmlChildrenNode->name<<endl;
                if (!xmlStrcmp(cur->name, (const xmlChar *) "slideshow"))
                    parseSlideShow(doc,cur);
                else
		if ((!xmlStrcmp(cur->name, (const xmlChar *) "base")) || 
                    (!xmlStrcmp(cur->name, (const xmlChar *) "page")))
            		parseSlide(doc,cur);
                else
                    cerr<<"Only \"slideshow\", \"base\" and \"page\" elements allowed at this level"<<endl;
		cur=cur->next;
	}
	*/
	
	xmlFreeDoc(doc);
//	return pres;
 }

 
 Presentation* XmlLoader::parseSlideShow(xmlDocPtr doc, xmlNodePtr cur, Presentation *maybe_presentation){
  
  xmlChar* videoMode = xmlGetProp(cur, (const xmlChar *) "video_mode");
  

  Presentation *pres = maybe_presentation;
  if (pres == NULL) {
	  int mode = VIDEO_DEFAULT;
  
	  if(!xmlStrcmp(videoMode,(const xmlChar *) "VIDEO_ASCII"))
		  mode = VIDEO_ASCII;
	  
	  cout<<"<	slideshow video_mode=\""<<videoMode<<"\">"<<endl;   
	  pres = new  Presentation(mode);
  }
  
 // cout<<"TODO:  Create Presentation"<<endl;
  cur=cur->xmlChildrenNode;
  
  while(cur != NULL)
  {
    if ((!xmlStrcmp(cur->name, (const xmlChar *) "base")) || 
        (!xmlStrcmp(cur->name, (const xmlChar *) "page")))
	    parseSlide(doc,cur,pres);
    else if (!xmlStrcmp(cur->name, (const xmlChar *) "font"))
        parseFont(doc,cur,pres);
/*    else
        cerr<<"Only \"font\", \"base\" and \"page\" elements allowed at this level, found ("<<cur->name<<")"<<endl;
  */
//      cout<<cur->name<<endl;
    cur=cur->next;      
  }//while(cur != NULL)
  return pres;
 }//void parseSlideShow(xmlDocPtr doc, xmlNodePtr cur)
 
 
 
void  XmlLoader::parseFont(xmlDocPtr doc, xmlNodePtr cur,Presentation* pres)
 {
     //cur=cur->xmlChildrenNode;
   
	 const char * n = (const char *) xmlGetProp(cur, (const xmlChar *) "name");
	 const char * fileName =  (const char *) xmlGetProp(cur, (const xmlChar *) "file_name");
	 const char * strSize =  (const char *) xmlGetProp(cur, (const xmlChar *) "size");
	 int size = char2int(strSize);
	 cout<<"Font ["<<n<<","<<fileName<<","<<size<<"] .."<<endl;     
	 pres->Fonts.Load(n,fileName,size);
	 cout<<"Font ["<<n<<","<<fileName<<","<<size<<"] loaded"<<endl;
	 return;
 }
 
 
 
 
 
 void  XmlLoader::parseSlide(xmlDocPtr doc, xmlNodePtr cur,Presentation* pres) {

 cout<<"Parsing slide ..."<<endl;
  //cout<<"title: "<<xmlGetProp(cur, (const xmlChar *) "title")<<endl;
 cout<<"slide type is : "<<cur->name<<endl;

 Page * page = NULL;
 char * type = (char *) cur->name;
 
 if(!strcmp(type,"base")) {
        char * aux = (char *)xmlGetProp(cur,(const xmlChar*)"ID");
        cout<<"BASE:Found, ID = ["<<aux<<"]"<<endl;
        page = new Page();
        if(aux)
            templates[aux] = page;
 }
else{
	cout<<"PAGE:TODO: Check if page uses a base, if so, add from vector"<<endl;
        char * aux = (char *)xmlGetProp(cur,(const xmlChar*)"templateID");
        Page *base;
        if (aux)
            base = templates[aux];

        page = pres->AddNewPage(base);
}
 
lastDrawable = NULL;

 /*if (!(xmlStrcmp(cur->name, (const xmlChar *)"base"))){
 cout<<"TODO: Add base to base vector (must be cleaned at end)"<<endl;
 }else{
    cout<<"TODO: Check if page uses a base, if so, add from vector"<<endl;
 }  */     
     
 cur=cur->xmlChildrenNode;
 while(cur != NULL) {
     
    if (!(xmlStrcmp(cur->name, (const xmlChar *)"writer")))
        parseWriter(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"image")))
            parseImage(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"image_scrolled")))
            parseImageScrolled(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"mpeg")))
            parseMpeg(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"mpeg_scrolled")))
             parseMpegScrolled(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"writer_text")))
            parseWriterText(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"slide_text")))
            parseSlideText(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"text_scrolled")))
             parseTextScrolled(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"text_double")))
            parseTextDouble(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"text_double_scrolled")))
            parseTextDoubleScrolled(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"stop")))
            parseStop(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"stars")))
            parseStars(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"plasma")))
            parsePlasma(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"fires")))
            parseFires(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"eraser")))
            parseEraser(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"gradient")))
            parseGradient(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"effect_darkness")))
            parseEffectDarkness(doc,cur,page);
    else if (!(xmlStrcmp(cur->name, (const xmlChar *)"spinner")))
            parseSpinner(doc,cur,page);    
 
    parseHAlign(cur, lastDrawable);
    parseVAlign(cur, lastDrawable);
        /*if( xmlNodeListGetString(doc, cur->xmlChildrenNode,1) != NULL)	 	
            cout<<"text : ["<< xmlNodeListGetString(doc, cur->xmlChildrenNode,1)<<"]"<<endl;
        */
    cur=cur->next;
 }//while (cur != NULL)
 return;
 }

 
 
 
 void XmlLoader::parseWriter (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * font_name = (char *)   xmlGetProp(cur, (const xmlChar *) "font_name");
 char *aux = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
 int  x,y;
 
 parseXY(cur,x,y,SS_UNDEFINED_COLOR);

 int color1 = SS_UNDEFINED_COLOR;
 if(aux)
 	color1=char2int(aux);
	
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "color2");
 int color2 = SS_UNDEFINED_COLOR;
 if(aux)
 	color2 = char2int(aux);
 aux = NULL;
 
 cout<<"Writer ["<<font_name<<","<<x<<","<<y<<","<<color1<<","<<color2<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewWriter(font_name,x,y,color1,color2);
}
 
 
 
 
void XmlLoader::parseWriterText (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char *text =  (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode,1);

 cout<<"WriterText ["<<text<<"] loaded"<<endl;      
  
 *page+=text; 
 }
 
 
 
 
 
 void XmlLoader::parseImage(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * file_name = (char *)   xmlGetProp(cur, (const xmlChar *) "file_name");
 char * aux = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
 int x,y;
 
 parseXY(cur,x,y);
  
 aux = NULL;

 cout<<"Image ["<<file_name<<","<<x<<","<<y<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewImage(file_name,x,y); 
 }
 
 
 void XmlLoader::parseImageScrolled(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * file_name = (char *)   xmlGetProp(cur, (const xmlChar *) "file_name");
 
 int dir = -1;
 int another = -1;
 int stop_at = -1;

 parseScrolledCommon(cur,dir,another,stop_at);

 cout<<"ImageScrolled ["<<dir<<","<<another<<","<<stop_at<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewImageScrolled(file_name,dir,another,stop_at); 
 } 
 
 
 
 void XmlLoader::parseXY(xmlNodePtr cur, int &x, int &y, int defaultValue)
 {
 	x =defaultValue;
	y = defaultValue;
	
	char * aux  = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
	if(aux)
	    x = char2int(aux);
	  
	aux  = (char *)   xmlGetProp(cur, (const xmlChar *) "y");
 
	if(aux)
	    y = char2int(aux); 
 
 }
 

 char* XmlLoader::parseHAlign(xmlNodePtr cur, slideshow::Drawable *drawable)
 {
  char *aux = (char *) xmlGetProp(cur,(const xmlChar *) "halign");

  if(aux && drawable){
    if(!strcmp(aux,"LEFT"))
        drawable->AlignLeft();
    else if(!strcmp(aux,"RIGTH"))
        drawable->AlignRight();
    else if(!strcmp(aux,"CENTER"))
        drawable->AlignHorizontalCenter();
    cout<<"HALIGN loaded ("<<aux<<")"<<endl;
  } 
     return aux;
 }
 
 
 
char* XmlLoader::parseVAlign(xmlNodePtr cur, slideshow::Drawable *drawable)
 {
  char *aux = (char *) xmlGetProp(cur,(const xmlChar *) "valign");

  if(aux && drawable){
     if(!strcmp(aux,"UP"))
        drawable->AlignUp();
    else if(!strcmp(aux,"DOWN"))
        drawable->AlignDown();
    else if(!strcmp(aux,"CENTER"))
        drawable->AlignVerticalCenter();

    cout<<"VALIGN loaded ("<<aux<<")"<<endl;
  } 
     return aux;
 } 
 
 
 
 
 
 void XmlLoader::parseScrolledCommon(xmlNodePtr cur, int &dir, int &another, int &stop_at) {
 	char *aux = (char *)   xmlGetProp(cur, (const xmlChar *) "dir");

	another = -1;
	stop_at = -1;
	
	if(aux)	{
	    if(!strcmp(aux,"SOUTH_NORTH"))
		dir = SOUTH_NORTH;
	    else if(!strcmp(aux,"NORTH_SOUTH"))
		dir = NORTH_SOUTH;
	    else if(!strcmp(aux,"EAST_WEST"))
		dir = EAST_WEST;
	    else if(!strcmp(aux,"WEST_EAST"))
		dir = WEST_EAST;
	   else
	   	dir = NONE;
	}
	
	aux = (char *)   xmlGetProp(cur, (const xmlChar *) "another");
	if(aux)
		another = char2int(aux);	
 	aux = (char *)   xmlGetProp(cur, (const xmlChar *) "stop_at");
	if(aux)
		stop_at = char2int(aux);	 	
 }
 

 
 void XmlLoader::parseMpeg(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * file_name = (char *)   xmlGetProp(cur, (const xmlChar *) "file_name");
 char * aux = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
 int x,y;
 
  parseXY(cur,x,y);
  

 if(aux)
     y = char2int(aux);
 aux = NULL;

 cout<<"Mpeg ["<<file_name<<","<<x<<","<<y<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewMpeg(file_name,x,y); 
 }  
 
 
 
 void XmlLoader::parseMpegScrolled(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * file_name = (char *)   xmlGetProp(cur, (const xmlChar *) "file_name");
 
 int dir = -1;
 int another = -1;
 int stop_at = -1;

 parseScrolledCommon(cur,dir,another,stop_at);

 cout<<"MpegScrolled ["<<dir<<","<<another<<","<<stop_at<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewMpegScrolled(file_name,dir,another,stop_at); 
 }  
 
 
 
 
 

 void XmlLoader::parseSlideText (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * font_name = (char *)   xmlGetProp(cur, (const xmlChar *) "font_name");
 char *aux = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
 int  x,y;

 parseXY(cur,x,y);
 
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "color");
 int color = 0;
  if(aux)
     color = char2int(aux);
 aux = NULL;
 
 char *text =  (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode,1);

 cout<<"SlideText ["<<font_name<<","<<text<<","<<x<<","<<y<<","<<color<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewText(font_name,text,x,y,color);

  
 }
 

  void XmlLoader::parseTextScrolled(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * font_name = (char *)   xmlGetProp(cur, (const xmlChar *) "font_name");
 
 int dir = -1;
 int another = -1;
 int stop_at = -1;
 int color = 0x000000;
 
 parseScrolledCommon(cur,dir,another,stop_at);

 char *aux = (char *) xmlGetProp(cur,(const xmlChar *)"color");
 if(aux)
   color = char2int(aux);
   
 char *text =  (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode,1);
 
 cout<<"TextScrolled ["<<font_name<<","<<text<<","<<dir<<","<<another<<","<<stop_at<<","<<color<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewTextScrolled(font_name,text,dir,another,stop_at,color);  
  
 }
 
 
 
  
 void XmlLoader::parseTextDouble (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char * font_name = (char *)   xmlGetProp(cur, (const xmlChar *) "font_name");
 char *aux = (char *)   xmlGetProp(cur, (const xmlChar *) "x");
 int x,y;
 
 parseXY(cur,x,y);
 
 
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "color1");
 int color1 = 0x000000;
  if(aux)
     color1 = char2int(aux);
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "color2");
 int color2 = 0xFFFFFF;
  if(aux)
     color2 = char2int(aux);
 aux = NULL;
 
 char *text =  (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode,1);

 cout<<"SlideText ["<<font_name<<","<<text<<","<<x<<","<<y<<","<<color1<<","<<color2<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewTextDouble(font_name,text,x,y,color1,color2);
 
 }
 
 
 
 void XmlLoader::parseTextDoubleScrolled (xmlDocPtr doc, xmlNodePtr cur,Page* page){

 char * font_name = (char *)   xmlGetProp(cur, (const xmlChar *) "font_name");
 
 int dir = -1;
 int another = -1;
 int stop_at = -1;
 int color1 = 0x000000;
 int color2 = 0xFFFFFF;
 
 parseScrolledCommon(cur,dir,another,stop_at);
 
 char *aux = (char *) xmlGetProp(cur,(const xmlChar *)"color1");
 
 if(aux)
   color1 = char2int(aux);
   
aux = (char *) xmlGetProp(cur,(const xmlChar *)"color2");
 if(aux)
   color2 = char2int(aux);   
   
 char *text =  (char *) xmlNodeListGetString(doc, cur->xmlChildrenNode,1);
 
 cout<<"TextDoubleScrolled ["<<font_name<<","<<text<<","<<dir<<","<<another<<","<<stop_at<<","<<color1<<","<<color2<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewTextDoubleScrolled(font_name,text,dir,another,stop_at,color1,color2);    
 }
 
 
 void XmlLoader::parseStop (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char* aux = (char *)   xmlGetProp(cur, (const xmlChar *) "secs");
 float secs = 0;
 if(aux)
 	secs = atof(aux);

aux = NULL;

 cout<<"Stop ["<<secs<<"] loaded"<<endl;      
  
 page->AddStop(secs);
 
 }
 
 
 
  void XmlLoader::parseStars (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
  cout<<"Stars loaded"<<endl;
  
 lastDrawable = page->AddNewStars();
 
 }

 
 void XmlLoader::parsePlasma (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
  cout<<"Plasma loaded"<<endl;
  
 lastDrawable = page->AddNewPlasma();
 
 } 
 
 
  void XmlLoader::parseFires (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
  cout<<"Fires loaded"<<endl;
  
 lastDrawable = page->AddNewFires();
 
 }  
 
 
 void XmlLoader::parseEraser (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
  cout<<"Eraser loaded"<<endl;
  char * aux = (char*) xmlGetProp(cur,(const xmlChar *) "color");
  int color = 0;
  if(aux)
  	color=char2int(aux);
  
 lastDrawable = page->AddNewEraser();
 
 }
 
 
 void XmlLoader::parseEffectDarkness(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 cout<<"EffectDarkness, lastDrawable -->"<<lastDrawable<<endl;
	
 if(!lastDrawable)
 	cout<<"EffectDarkness couldn't be applied as no Drawable was set before its invocation"<<endl;
else
{
	cout<<"EffectDarkness loaded"<<endl;
	page->AddNewEffectDarkness(lastDrawable);

} 
 }
 
 
 
 
void XmlLoader::parseSpinner(xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 cout<<"Spinner, lastDrawable -->"<<lastDrawable<<endl;
	
 if(!lastDrawable)
 	cout<<"Spinner couldn't be applied as no Drawable was set before its invocation"<<endl;
else
{
        int degrees_second = 90;
        char *aux =  (char *)   xmlGetProp(cur, (const xmlChar *) "degrees_second");
        if(aux)
            degrees_second = char2int(aux,degrees_second);
        
	cout<<"Spinner loaded"<<endl;
	page->AddNewSpinner(lastDrawable,degrees_second);

} 
 } 
 
 
 
 
 void XmlLoader::parseGradient (xmlDocPtr doc, xmlNodePtr cur,Page* page){
 
 char *aux =  (char *)   xmlGetProp(cur, (const xmlChar *) "color1");
 unsigned int color1 = 0;
  if(aux)
     color1 = char2int(aux);
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "color2");
 unsigned int color2 = 0;
  if(aux)
     color2 = char2int(aux);
 aux = NULL;
 
 
 SlideShowGradient type = GRADIENT_VERTICAL;
 
 aux = (char *)   xmlGetProp(cur, (const xmlChar *) "type");
 if(aux && !strcmp(aux,"GRADIENT_HORIZONTAL"))
    type = GRADIENT_HORIZONTAL;
 
  cout<<"Gradient ["<<color1<<","<<color2<<"] loaded"<<endl;      
  
 lastDrawable = page->AddNewGradient(color1,color2,type);
 
 }
 
 
 int XmlLoader::char2int(const char * strNumber, int defaultValue)
 {
     int val = 0;   
     int state = 0;
     if(strNumber)
        state = sscanf(strNumber,"%i",&val);
     
     val = state ? val : defaultValue;
     
     return val;
 }
 
