/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef __XmlLoader__
#define __XmlLoader__

#include <iostream>
#include <vector>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "Loader.h"
#include "Presentation.h"

using namespace std;

#ifdef HAVE_GNU_CXX_HASHMAP
typedef __gnu_cxx::hash_map<std::string, Page*, ures_hasher, ures_eqstr> Templates_HashMap;
#else
typedef std::hash_map<std::string, Page*, ures_hasher, ures_eqstr> Templates_HashMap;
#endif



class XmlLoader : public Loader {
public:
	   XmlLoader();
	      
	   virtual ~XmlLoader();
   
	   virtual void LoadInto (const char *filename, Presentation *load_into);
	   
           //TODO: MAKE THIS METHOD STATIC 
           int char2int(const char * strNumber, int defaultValue = -1);
           
protected:

	Templates_HashMap templates;
	   
	
	   
/** Stores the last drawable object added to the current slide.
     It is set to Null when entering parseSlide.
     Used to apply effects like effectDarkness.
    */
slideshow::Drawable * lastDrawable;

Presentation* parseSlideShow(xmlDocPtr doc, xmlNodePtr cur, Presentation *maybe_presentation=NULL);

void parseFont(xmlDocPtr doc, xmlNodePtr cur,Presentation *pres);

void parseSlide(xmlDocPtr doc, xmlNodePtr cur,Presentation *pres);

void parseWriter (xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseWriterText (xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseImage(xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseImageScrolled(xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseScrolledCommon(xmlNodePtr cur, int &dir, int &another, int &stop_at);

void parseXY (xmlNodePtr cur, int &x, int &y, int defaultValue = -1);

void parseMpeg(xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseMpegScrolled(xmlDocPtr doc, xmlNodePtr cur,Page* page);


void parseSlideText (xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseTextScrolled (xmlDocPtr doc, xmlNodePtr cur,Page* page);
 
void parseTextDouble (xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseTextDoubleScrolled (xmlDocPtr doc, xmlNodePtr cur,Page* page);
 
char* parseHAlign(xmlNodePtr cur, slideshow::Drawable* drawable);

char* parseVAlign(xmlNodePtr cur, slideshow::Drawable* drawable);

void parseStop (xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseStars (xmlDocPtr doc, xmlNodePtr cur,Page* page);
  
void parsePlasma (xmlDocPtr doc, xmlNodePtr cur,Page* page);
 
void parseFires (xmlDocPtr doc, xmlNodePtr cur,Page* page);
 
void parseEraser (xmlDocPtr doc, xmlNodePtr cur,Page* page);
  
 
void parseEffectDarkness(xmlDocPtr doc, xmlNodePtr cur,Page* page);

void parseSpinner(xmlDocPtr doc, xmlNodePtr cur,Page* page);

 
void parseGradient (xmlDocPtr doc, xmlNodePtr cur,Page* page);
};

#endif // __XmlLoader__
