/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <unistd.h>
#include <iostream>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Utilities.h"
#include "Presentation.h"
#include "AutoSlide.h"

#include "HtmlExporter.h"


using namespace std;


void
print_usage (void)
{
	printf ("Usage:\n"	
		"\t slideshow [-e [prefix]] presentation\n"
		"\t slideshow [-e [prefix]] [-r] -a [directory]\n\n"
		"-h [prefix] \t Export to HTML\n"
		"-p [prefix] \t Export to Python\n"
		"-x [prefix] \t Export to XML\n"
		"-c [prefix] \t Export to C++\n"
		"-a          \t Autoslide\n"
		"-r          \t Recursive\n"
		"-f          \t FullScreen mode\n"
		"-w          \t Enable hardware acceleration\n"
		"\n");
}

char current_dir_buf[1024];
char *current_dir = NULL;

int
main (int argc, char **argv)
{
	if (argc < 2) {
		print_usage();
		exit(1);
	}

	bool  autoslide = false;;
	char *autoslide_dir = NULL;
	bool  autoslide_recursive = false;
	bool  hardware_accel = false;
	bool  fullscreen_mode = false;
	
	bool export_cpp    = false;
	bool export_xml    = false;
	bool export_html   = false;
	bool  export_python      = false;
	char* export_python_file = NULL;
	
	int ch;
	while ((ch = getopt(argc, argv, "arhp:xcfw")) != -1) {
		   switch(ch) {
		   case 'c':
			   export_cpp = true;
			   break;
		   case 'h':
			   export_html = true;
			   break;
		   case 'p':
			   export_python = true;
			   export_python_file = optarg;
			   break;
		   case 'x':
			   export_xml = true;
			   break;
		   case 'r':
			   autoslide_recursive = true;
			   break;
		   case 'a': {
			   autoslide = true;
			   autoslide_dir = optarg;			   
			   break;
		   }
		   case 'f':
			   fullscreen_mode = true;
			   break;
		   case 'w':
			   hardware_accel = true;
			   break;
		   case '?':
			   cerr << endl << "ERROR: Unknown option '" << (char)ch << "'" << endl;
			   print_usage();
			   exit(1);
			   break;
		   default:
			   print_usage();
			   exit(1);
		   }
	   }

	   Presentation *p;
	   char         *arg = argv[optind];
	   string        dir = arg? getDirName(arg) : ".";
	   int           flags = 0;
	   
	   // Get current dir
	   current_dir = getcwd (&current_dir_buf[0], 1024);
	   chdir(dir.c_str());

	   // SDL Flags
	   if (hardware_accel) {
		   flags |= SDL_HWSURFACE|SDL_HWACCEL;
	   }	   
	   if (fullscreen_mode) {
		   flags |= SDL_FULLSCREEN;
	   }
		   
	   
	   if (autoslide) {
		   p = new AutoSlide();
		   static_cast<AutoSlide*>(p)->Load(dir.c_str(), autoslide_recursive);		   
	   } else {
		   p = new Presentation(flags);
		   p->Load(arg);
	   }

	   if (export_html || export_python || export_xml) {
		   string prefix = (char *)current_dir;
		   prefix += "/";

		   if (export_html) {
			   p->ExportHTML (prefix.c_str());
		   }

		   if (export_python) {
			   prefix += export_python_file;
			   p->ExportPython (prefix.c_str());
		   }

		   if (export_xml) {
			   p->ExportXML (prefix.c_str());
		   }
	   } else {
		   p->Run();
	   }
	   
	   delete p;
	   chdir (current_dir);
	   
	   return 0;
}
